@@ -21,12 +21,10 @@ DATETIME_INPUT_FORMATS = [
     '%d.%m.%Y %H.%M.%S',     # '20.3.2014 14.30.59'
     '%d.%m.%Y %H.%M.%S.%f',  # '20.3.2014 14.30.59.000200'
     '%d.%m.%Y %H.%M',        # '20.3.2014 14.30'
-    '%d.%m.%Y',              # '20.3.2014'
 
     '%d.%m.%y %H.%M.%S',     # '20.3.14 14.30.59'
     '%d.%m.%y %H.%M.%S.%f',  # '20.3.14 14.30.59.000200'
     '%d.%m.%y %H.%M',        # '20.3.14 14.30'
-    '%d.%m.%y',              # '20.3.14'
 ]
 TIME_INPUT_FORMATS = [
     '%H.%M.%S',     # '14.30.59'
