@@ -16,31 +16,26 @@ FIRST_DAY_OF_WEEK = 1  # (Monday)
 # Kept ISO formats as they are in first position
 DATE_INPUT_FORMATS = [
     '%Y-%m-%d', '%m/%d/%Y', '%m/%d/%y',     # '2006-10-25', '10/25/2006', '10/25/06'
+    '%d.%m.%Y', '%d.%m.%y',                 # '25.10.2006', '25.10.06'
     # '%d %b %Y', '%d %b, %Y', '%d %b. %Y',   # '25 Oct 2006', '25 Oct, 2006', '25 Oct. 2006'
     # '%d %B %Y', '%d %B, %Y',                # '25 October 2006', '25 October, 2006'
-    # '%d.%m.%Y', '%d.%m.%y',                 # '25.10.2006', '25.10.06'
 ]
 DATETIME_INPUT_FORMATS = [
     '%Y-%m-%d %H:%M:%S',     # '2006-10-25 14:30:59'
     '%Y-%m-%d %H:%M:%S.%f',  # '2006-10-25 14:30:59.000200'
     '%Y-%m-%d %H:%M',        # '2006-10-25 14:30'
-    '%Y-%m-%d',              # '2006-10-25'
     '%d.%m.%Y %H:%M:%S',     # '25.10.2006 14:30:59'
     '%d.%m.%Y %H:%M:%S.%f',  # '25.10.2006 14:30:59.000200'
     '%d.%m.%Y %H:%M',        # '25.10.2006 14:30'
-    '%d.%m.%Y',              # '25.10.2006'
     '%d.%m.%y %H:%M:%S',     # '25.10.06 14:30:59'
     '%d.%m.%y %H:%M:%S.%f',  # '25.10.06 14:30:59.000200'
     '%d.%m.%y %H:%M',        # '25.10.06 14:30'
-    '%d.%m.%y',              # '25.10.06'
     '%m/%d/%Y %H:%M:%S',     # '10/25/2006 14:30:59'
     '%m/%d/%Y %H:%M:%S.%f',  # '10/25/2006 14:30:59.000200'
     '%m/%d/%Y %H:%M',        # '10/25/2006 14:30'
-    '%m/%d/%Y',              # '10/25/2006'
     '%m/%d/%y %H:%M:%S',     # '10/25/06 14:30:59'
     '%m/%d/%y %H:%M:%S.%f',  # '10/25/06 14:30:59.000200'
     '%m/%d/%y %H:%M',        # '10/25/06 14:30'
-    '%m/%d/%y',              # '10/25/06'
 ]
 DECIMAL_SEPARATOR = '.'
 THOUSAND_SEPARATOR = " "
