@@ -190,7 +190,7 @@ class UserAttributeSimilarityValidator:
                     )
 
     def get_help_text(self):
-        return _("Your password can't be too similar to your other personal information.")
+        return _('Your password can’t be too similar to your other personal information.')
 
 
 class CommonPasswordValidator:
@@ -207,13 +207,11 @@ class CommonPasswordValidator:
 
     def __init__(self, password_list_path=DEFAULT_PASSWORD_LIST_PATH):
         try:
-            with gzip.open(str(password_list_path)) as f:
-                common_passwords_lines = f.read().decode().splitlines()
-        except IOError:
-            with open(str(password_list_path)) as f:
-                common_passwords_lines = f.readlines()
-
-        self.passwords = {p.strip() for p in common_passwords_lines}
+            with gzip.open(password_list_path, 'rt', encoding='utf-8') as f:
+                self.passwords = {x.strip() for x in f}
+        except OSError:
+            with open(password_list_path) as f:
+                self.passwords = {x.strip() for x in f}
 
     def validate(self, password, user=None):
         if password.lower().strip() in self.passwords:
@@ -223,7 +221,7 @@ class CommonPasswordValidator:
             )
 
     def get_help_text(self):
-        return _("Your password can't be a commonly used password.")
+        return _('Your password can’t be a commonly used password.')
 
 
 class NumericPasswordValidator:
@@ -238,4 +236,4 @@ class NumericPasswordValidator:
             )
 
     def get_help_text(self):
-        return _("Your password can't be entirely numeric.")
+        return _('Your password can’t be entirely numeric.')
