@@ -14,10 +14,9 @@ class SpatialProxy(DeferredAttribute):
         Initialize on the given Geometry or Raster class (not an instance)
         and the corresponding field.
         """
-        self._field = field
         self._klass = klass
         self._load_func = load_func or klass
-        super().__init__(field.attname)
+        super().__init__(field)
 
     def __get__(self, instance, cls=None):
         """
@@ -32,7 +31,7 @@ class SpatialProxy(DeferredAttribute):
 
         # Getting the value of the field.
         try:
-            geo_value = instance.__dict__[self._field.attname]
+            geo_value = instance.__dict__[self.field.attname]
         except KeyError:
             geo_value = super().__get__(instance, cls)
 
@@ -44,7 +43,7 @@ class SpatialProxy(DeferredAttribute):
             # Otherwise, a geometry or raster object is built using the field's
             # contents, and the model's corresponding attribute is set.
             geo_obj = self._load_func(geo_value)
-            setattr(instance, self._field.attname, geo_obj)
+            setattr(instance, self.field.attname, geo_obj)
         return geo_obj
 
     def __set__(self, instance, value):
@@ -56,7 +55,7 @@ class SpatialProxy(DeferredAttribute):
         To set rasters, use JSON or dict values.
         """
         # The geographic type of the field.
-        gtype = self._field.geom_type
+        gtype = self.field.geom_type
 
         if gtype == 'RASTER' and (value is None or isinstance(value, (str, dict, self._klass))):
             # For raster fields, assure input is None or a string, dict, or
@@ -67,7 +66,7 @@ class SpatialProxy(DeferredAttribute):
             # general GeometryField is used.
             if value.srid is None:
                 # Assigning the field SRID if the geometry has no SRID.
-                value.srid = self._field.srid
+                value.srid = self.field.srid
         elif value is None or isinstance(value, (str, memoryview)):
             # Set geometries with None, WKT, HEX, or WKB
             pass
@@ -76,5 +75,5 @@ class SpatialProxy(DeferredAttribute):
                 instance.__class__.__name__, gtype, type(value)))
 
         # Setting the objects dictionary with the value, and returning.
-        instance.__dict__[self._field.attname] = value
+        instance.__dict__[self.field.attname] = value
         return value
