@@ -1,5 +1,6 @@
 from itertools import chain
 
+from django.utils.inspect import func_accepts_kwargs
 from django.utils.itercompat import is_iterable
 
 
@@ -8,12 +9,15 @@ class Tags:
     Built-in tags for internal checks.
     """
     admin = 'admin'
+    async_support = 'async_support'
     caches = 'caches'
     compatibility = 'compatibility'
     database = 'database'
     models = 'models'
     security = 'security'
     signals = 'signals'
+    sites = 'sites'
+    staticfiles = 'staticfiles'
     templates = 'templates'
     translation = 'translation'
     urls = 'urls'
@@ -35,13 +39,17 @@ class CheckRegistry:
 
             registry = CheckRegistry()
             @registry.register('mytag', 'anothertag')
-            def my_check(apps, **kwargs):
+            def my_check(app_configs, **kwargs):
                 # ... perform checks and collect `errors` ...
                 return errors
             # or
             registry.register(my_check, 'mytag', 'anothertag')
         """
         def inner(check):
+            if not func_accepts_kwargs(check):
+                raise TypeError(
+                    'Check functions must accept keyword arguments (**kwargs).'
+                )
             check.tags = tags
             checks = self.deployment_checks if kwargs.get('deploy') else self.registered_checks
             checks.add(check)
@@ -54,7 +62,7 @@ class CheckRegistry:
                 tags += (check,)
             return inner
 
-    def run_checks(self, app_configs=None, tags=None, include_deployment_checks=False):
+    def run_checks(self, app_configs=None, tags=None, include_deployment_checks=False, databases=None):
         """
         Run all registered checks and return list of Errors and Warnings.
         """
@@ -63,13 +71,9 @@ class CheckRegistry:
 
         if tags is not None:
             checks = [check for check in checks if not set(check.tags).isdisjoint(tags)]
-        else:
-            # By default, 'database'-tagged checks are not run as they do more
-            # than mere static code analysis.
-            checks = [check for check in checks if Tags.database not in check.tags]
 
         for check in checks:
-            new_errors = check(app_configs=app_configs)
+            new_errors = check(app_configs=app_configs, databases=databases)
             assert is_iterable(new_errors), (
                 "The function %r did not return a list. All functions registered "
                 "with the checks registry must return a list." % check)
