@@ -1,25 +1,10 @@
 import os
 import tempfile
 from os.path import abspath, dirname, join, normcase, sep
+from pathlib import Path
 
 from django.core.exceptions import SuspiciousFileOperation
 
-# For backwards-compatibility in Django 2.0
-abspathu = abspath
-
-
-def upath(path):
-    """Always return a unicode path (did something for Python 2)."""
-    return path
-
-
-def npath(path):
-    """
-    Always return a native path, that is unicode on Python 3 and bytestring on
-    Python 2. Noop for Python 3.
-    """
-    return path
-
 
 def safe_join(base, *paths):
     """
@@ -63,3 +48,12 @@ def symlinks_supported():
         except (OSError, NotImplementedError):
             supported = False
         return supported
+
+
+def to_path(value):
+    """Convert value to a pathlib.Path instance, if not already a Path."""
+    if isinstance(value, Path):
+        return value
+    elif not isinstance(value, str):
+        raise TypeError('Invalid path type: %s' % type(value).__name__)
+    return Path(value)
