@@ -31,6 +31,11 @@ class Book(models.Model):
     is_best_seller = models.BooleanField(default=0, null=True)
     is_best_seller2 = models.NullBooleanField(default=0)
     date_registered = models.DateField(null=True)
+    availability = models.BooleanField(choices=(
+        (False, 'Paid'),
+        (True, 'Free'),
+        (None, 'Obscure'),
+    ), null=True)
     # This field name is intentionally 2 characters long (#16080).
     no = models.IntegerField(verbose_name='number', blank=True, null=True)
 
@@ -38,6 +43,10 @@ class Book(models.Model):
         return self.title
 
 
+class ImprovedBook(models.Model):
+    book = models.OneToOneField(Book, models.CASCADE)
+
+
 class Department(models.Model):
     code = models.CharField(max_length=4, unique=True)
     description = models.CharField(max_length=50, blank=True, null=True)
