@@ -1,16 +1,17 @@
 from django import forms
 from django.contrib import admin
+from django.core.exceptions import ValidationError
 from django.db import models
 
 from .models import (
     Author, BinaryTree, CapoFamiglia, Chapter, Child, ChildModel1, ChildModel2,
-    Consigliere, EditablePKBook, ExtraTerrestrial, Fashionista, Holder,
-    Holder2, Holder3, Holder4, Inner, Inner2, Inner3, Inner4Stacked,
-    Inner4Tabular, NonAutoPKBook, NonAutoPKBookChild, Novel,
-    NovelReadonlyChapter, ParentModelWithCustomPk, Poll, Profile,
-    ProfileCollection, Question, ReadOnlyInline, ShoppingWeakness, Sighting,
-    SomeChildModel, SomeParentModel, SottoCapo, Teacher, Title,
-    TitleCollection,
+    Consigliere, EditablePKBook, ExtraTerrestrial, Fashionista, FootNote,
+    Holder, Holder2, Holder3, Holder4, Holder5, Inner, Inner2, Inner3,
+    Inner4Stacked, Inner4Tabular, Inner5Stacked, Inner5Tabular, NonAutoPKBook,
+    NonAutoPKBookChild, Novel, NovelReadonlyChapter, OutfitItem,
+    ParentModelWithCustomPk, Person, Poll, Profile, ProfileCollection,
+    Question, ReadOnlyInline, ShoppingWeakness, Sighting, SomeChildModel,
+    SomeParentModel, SottoCapo, Teacher, Title, TitleCollection,
 )
 
 site = admin.AdminSite(name="admin")
@@ -102,7 +103,7 @@ class TitleForm(forms.ModelForm):
         title1 = cleaned_data.get("title1")
         title2 = cleaned_data.get("title2")
         if title1 != title2:
-            raise forms.ValidationError("The two titles must be the same")
+            raise ValidationError("The two titles must be the same")
         return cleaned_data
 
 
@@ -126,11 +127,54 @@ class Holder4Admin(admin.ModelAdmin):
     inlines = [Inner4StackedInline, Inner4TabularInline]
 
 
+class Inner5StackedInline(admin.StackedInline):
+    model = Inner5Stacked
+    classes = ('collapse',)
+
+
+class Inner5TabularInline(admin.TabularInline):
+    model = Inner5Tabular
+    classes = ('collapse',)
+
+
+class Holder5Admin(admin.ModelAdmin):
+    inlines = [Inner5StackedInline, Inner5TabularInline]
+
+
 class InlineWeakness(admin.TabularInline):
     model = ShoppingWeakness
     extra = 1
 
 
+class WeaknessForm(forms.ModelForm):
+    extra_field = forms.CharField()
+
+    class Meta:
+        model = ShoppingWeakness
+        fields = '__all__'
+
+
+class WeaknessInlineCustomForm(admin.TabularInline):
+    model = ShoppingWeakness
+    form = WeaknessForm
+
+
+class FootNoteForm(forms.ModelForm):
+    extra_field = forms.CharField()
+
+    class Meta:
+        model = FootNote
+        fields = '__all__'
+
+
+class FootNoteNonEditableInlineCustomForm(admin.TabularInline):
+    model = FootNote
+    form = FootNoteForm
+
+    def has_change_permission(self, request, obj=None):
+        return False
+
+
 class QuestionInline(admin.TabularInline):
     model = Question
     readonly_fields = ['call_me']
@@ -248,6 +292,14 @@ class TeacherAdmin(admin.ModelAdmin):
     inlines = [StudentInline]
 
 
+class AuthorTabularInline(admin.TabularInline):
+    model = Author
+
+
+class FashonistaStackedInline(admin.StackedInline):
+    model = Fashionista
+
+
 site.register(TitleCollection, inlines=[TitleInline])
 # Test bug #12561 and #12778
 # only ModelAdmin media
@@ -262,6 +314,7 @@ site.register(Novel, NovelAdmin)
 site.register(NovelReadonlyChapter, NovelReadonlyChapterAdmin)
 site.register(Fashionista, inlines=[InlineWeakness])
 site.register(Holder4, Holder4Admin)
+site.register(Holder5, Holder5Admin)
 site.register(Author, AuthorAdmin)
 site.register(CapoFamiglia, inlines=[ConsigliereInline, SottoCapoInline, ReadOnlyInlineInline])
 site.register(ProfileCollection, inlines=[ProfileInline])
@@ -271,3 +324,6 @@ site.register(ExtraTerrestrial, inlines=[SightingInline])
 site.register(SomeParentModel, inlines=[SomeChildModelInline])
 site.register([Question, Inner4Stacked, Inner4Tabular])
 site.register(Teacher, TeacherAdmin)
+site.register(Chapter, inlines=[FootNoteNonEditableInlineCustomForm])
+site.register(OutfitItem, inlines=[WeaknessInlineCustomForm])
+site.register(Person, inlines=[AuthorTabularInline, FashonistaStackedInline])
