@@ -1,8 +1,7 @@
-from __future__ import unicode_literals
-
 from django.contrib import admin
 from django.contrib.admin.options import ModelAdmin
 from django.contrib.auth.models import User
+from django.db.models import F
 from django.test import RequestFactory, TestCase
 
 from .models import (
@@ -11,17 +10,18 @@ from .models import (
 )
 
 
-class MockRequest(object):
+class MockRequest:
     pass
 
 
-class MockSuperUser(object):
-    def has_perm(self, perm):
+class MockSuperUser:
+    def has_perm(self, perm, obj=None):
         return True
 
     def has_module_perms(self, module):
         return True
 
+
 request = MockRequest()
 request.user = MockSuperUser()
 
@@ -34,9 +34,10 @@ class TestAdminOrdering(TestCase):
     in ModelAdmin rather that ordering defined in the model's inner Meta
     class.
     """
+    request_factory = RequestFactory()
 
-    def setUp(self):
-        self.request_factory = RequestFactory()
+    @classmethod
+    def setUpTestData(cls):
         Band.objects.bulk_create([
             Band(name='Aerosmith', bio='', rank=3),
             Band(name='Radiohead', bio='', rank=1),
@@ -50,7 +51,7 @@ class TestAdminOrdering(TestCase):
         """
         ma = ModelAdmin(Band, site)
         names = [b.name for b in ma.get_queryset(request)]
-        self.assertListEqual(['Aerosmith', 'Radiohead', 'Van Halen'], names)
+        self.assertEqual(['Aerosmith', 'Radiohead', 'Van Halen'], names)
 
     def test_specified_ordering(self):
         """
@@ -61,7 +62,14 @@ class TestAdminOrdering(TestCase):
             ordering = ('rank',)  # default ordering is ('name',)
         ma = BandAdmin(Band, site)
         names = [b.name for b in ma.get_queryset(request)]
-        self.assertListEqual(['Radiohead', 'Van Halen', 'Aerosmith'], names)
+        self.assertEqual(['Radiohead', 'Van Halen', 'Aerosmith'], names)
+
+    def test_specified_ordering_by_f_expression(self):
+        class BandAdmin(ModelAdmin):
+            ordering = (F('rank').desc(nulls_last=True),)
+        band_admin = BandAdmin(Band, site)
+        names = [b.name for b in band_admin.get_queryset(request)]
+        self.assertEqual(['Aerosmith', 'Van Halen', 'Radiohead'], names)
 
     def test_dynamic_ordering(self):
         """
@@ -73,10 +81,10 @@ class TestAdminOrdering(TestCase):
         request.user = super_user
         ma = DynOrderingBandAdmin(Band, site)
         names = [b.name for b in ma.get_queryset(request)]
-        self.assertListEqual(['Radiohead', 'Van Halen', 'Aerosmith'], names)
+        self.assertEqual(['Radiohead', 'Van Halen', 'Aerosmith'], names)
         request.user = other_user
         names = [b.name for b in ma.get_queryset(request)]
-        self.assertListEqual(['Aerosmith', 'Radiohead', 'Van Halen'], names)
+        self.assertEqual(['Aerosmith', 'Radiohead', 'Van Halen'], names)
 
 
 class TestInlineModelAdminOrdering(TestCase):
@@ -85,12 +93,13 @@ class TestInlineModelAdminOrdering(TestCase):
     define in InlineModelAdmin.
     """
 
-    def setUp(self):
-        self.band = Band.objects.create(name='Aerosmith', bio='', rank=3)
+    @classmethod
+    def setUpTestData(cls):
+        cls.band = Band.objects.create(name='Aerosmith', bio='', rank=3)
         Song.objects.bulk_create([
-            Song(band=self.band, name='Pink', duration=235),
-            Song(band=self.band, name='Dude (Looks Like a Lady)', duration=264),
-            Song(band=self.band, name='Jaded', duration=214),
+            Song(band=cls.band, name='Pink', duration=235),
+            Song(band=cls.band, name='Dude (Looks Like a Lady)', duration=264),
+            Song(band=cls.band, name='Jaded', duration=214),
         ])
 
     def test_default_ordering(self):
@@ -100,7 +109,7 @@ class TestInlineModelAdminOrdering(TestCase):
         """
         inline = SongInlineDefaultOrdering(self.band, site)
         names = [s.name for s in inline.get_queryset(request)]
-        self.assertListEqual(['Dude (Looks Like a Lady)', 'Jaded', 'Pink'], names)
+        self.assertEqual(['Dude (Looks Like a Lady)', 'Jaded', 'Pink'], names)
 
     def test_specified_ordering(self):
         """
@@ -108,14 +117,16 @@ class TestInlineModelAdminOrdering(TestCase):
         """
         inline = SongInlineNewOrdering(self.band, site)
         names = [s.name for s in inline.get_queryset(request)]
-        self.assertListEqual(['Jaded', 'Pink', 'Dude (Looks Like a Lady)'], names)
+        self.assertEqual(['Jaded', 'Pink', 'Dude (Looks Like a Lady)'], names)
 
 
 class TestRelatedFieldsAdminOrdering(TestCase):
-    def setUp(self):
-        self.b1 = Band.objects.create(name='Pink Floyd', bio='', rank=1)
-        self.b2 = Band.objects.create(name='Foo Fighters', bio='', rank=5)
+    @classmethod
+    def setUpTestData(cls):
+        cls.b1 = Band.objects.create(name='Pink Floyd', bio='', rank=1)
+        cls.b2 = Band.objects.create(name='Foo Fighters', bio='', rank=5)
 
+    def setUp(self):
         # we need to register a custom ModelAdmin (instead of just using
         # ModelAdmin) because the field creator tries to find the ModelAdmin
         # for the related model
@@ -131,9 +142,8 @@ class TestRelatedFieldsAdminOrdering(TestCase):
     def check_ordering_of_field_choices(self, correct_ordering):
         fk_field = site._registry[Song].formfield_for_foreignkey(Song.band.field, request=None)
         m2m_field = site._registry[Song].formfield_for_manytomany(Song.other_interpreters.field, request=None)
-
-        self.assertListEqual(list(fk_field.queryset), correct_ordering)
-        self.assertListEqual(list(m2m_field.queryset), correct_ordering)
+        self.assertEqual(list(fk_field.queryset), correct_ordering)
+        self.assertEqual(list(m2m_field.queryset), correct_ordering)
 
     def test_no_admin_fallback_to_model_ordering(self):
         # should be ordered by name (as defined by the model)
@@ -156,18 +166,18 @@ class TestRelatedFieldsAdminOrdering(TestCase):
         self.check_ordering_of_field_choices([self.b1, self.b2])
 
     def test_custom_queryset_still_wins(self):
-        """Test that custom queryset has still precedence (#21405)"""
+        """Custom queryset has still precedence (#21405)"""
         class SongAdmin(admin.ModelAdmin):
             # Exclude one of the two Bands from the querysets
             def formfield_for_foreignkey(self, db_field, request, **kwargs):
                 if db_field.name == 'band':
                     kwargs["queryset"] = Band.objects.filter(rank__gt=2)
-                return super(SongAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
+                return super().formfield_for_foreignkey(db_field, request, **kwargs)
 
             def formfield_for_manytomany(self, db_field, request, **kwargs):
                 if db_field.name == 'other_interpreters':
                     kwargs["queryset"] = Band.objects.filter(rank__gt=2)
-                return super(SongAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
+                return super().formfield_for_foreignkey(db_field, request, **kwargs)
 
         class StaticOrderingBandAdmin(admin.ModelAdmin):
             ordering = ('rank',)
