@@ -1,10 +1,8 @@
+from django.contrib import admin
 from django.db import models
-from django.utils import six
-from django.utils.encoding import python_2_unicode_compatible
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 
-@python_2_unicode_compatible
 class Site(models.Model):
     domain = models.CharField(max_length=100)
 
@@ -18,15 +16,22 @@ class Article(models.Model):
     """
     site = models.ForeignKey(Site, models.CASCADE, related_name="admin_articles")
     title = models.CharField(max_length=100)
-    hist = models.CharField(max_length=100, verbose_name=_("History"))
+    hist = models.CharField(
+        max_length=100,
+        verbose_name=_('History'),
+        help_text=_('History help text'),
+    )
     created = models.DateTimeField(null=True)
 
+    def __str__(self):
+        return self.title
+
     def test_from_model(self):
         return "nothing"
 
+    @admin.display(description='not What you Expect')
     def test_from_model_with_override(self):
         return "nothing"
-    test_from_model_with_override.short_description = "not What you Expect"
 
 
 class ArticleProxy(Article):
@@ -34,13 +39,12 @@ class ArticleProxy(Article):
         proxy = True
 
 
-@python_2_unicode_compatible
 class Count(models.Model):
     num = models.PositiveSmallIntegerField()
     parent = models.ForeignKey('self', models.CASCADE, null=True)
 
     def __str__(self):
-        return six.text_type(self.num)
+        return str(self.num)
 
 
 class Event(models.Model):
