@@ -31,3 +31,8 @@ class PlainAppsConfig(AppConfig):
 class RelabeledAppsConfig(AppConfig):
     name = 'apps'
     label = 'relabeled'
+
+
+class ModelPKAppsConfig(AppConfig):
+    name = 'apps'
+    default_auto_field = 'django.db.models.BigAutoField'
