@@ -1,3 +1,4 @@
+import gzip
 import os
 import sys
 import tempfile
@@ -17,22 +18,35 @@ from django.db import IntegrityError, connection
 from django.test import TestCase, TransactionTestCase, skipUnlessDBFeature
 
 from .models import (
-    Article, Category, NaturalKeyThing, PrimaryKeyUUIDModel, ProxySpy, Spy,
-    Tag, Visa,
+    Article, Category, CircularA, CircularB, NaturalKeyThing,
+    PrimaryKeyUUIDModel, ProxySpy, Spy, Tag, Visa,
 )
 
+try:
+    import bz2  # NOQA
+    HAS_BZ2 = True
+except ImportError:
+    HAS_BZ2 = False
+
+try:
+    import lzma  # NOQA
+    HAS_LZMA = True
+except ImportError:
+    HAS_LZMA = False
+
 
 class TestCaseFixtureLoadingTests(TestCase):
     fixtures = ['fixture1.json', 'fixture2.json']
 
-    def testClassFixtures(self):
+    def test_class_fixtures(self):
         "Test case has installed 3 fixture objects"
-        self.assertEqual(Article.objects.count(), 3)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Django conquers world!>',
-            '<Article: Copyright is fine the way it is>',
-            '<Article: Poker has no place on ESPN>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True), [
+                'Django conquers world!',
+                'Copyright is fine the way it is',
+                'Poker has no place on ESPN',
+            ],
+        )
 
 
 class SubclassTestCaseFixtureLoadingTests(TestCaseFixtureLoadingTests):
@@ -41,7 +55,7 @@ class SubclassTestCaseFixtureLoadingTests(TestCaseFixtureLoadingTests):
     """
     fixtures = []
 
-    def testClassFixtures(self):
+    def test_class_fixtures(self):
         "There were no fixture objects installed"
         self.assertEqual(Article.objects.count(), 0)
 
@@ -67,9 +81,26 @@ class DumpDataAssertMixin:
             primary_keys=primary_keys,
         )
         if filename:
-            with open(filename, "r") as f:
+            file_root, file_ext = os.path.splitext(filename)
+            compression_formats = {
+                '.bz2': (open, file_root),
+                '.gz': (gzip.open, filename),
+                '.lzma': (open, file_root),
+                '.xz': (open, file_root),
+                '.zip': (open, file_root),
+            }
+            if HAS_BZ2:
+                compression_formats['.bz2'] = (bz2.open, filename)
+            if HAS_LZMA:
+                compression_formats['.lzma'] = (lzma.open, filename)
+                compression_formats['.xz'] = (lzma.open, filename)
+            try:
+                open_method, file_path = compression_formats[file_ext]
+            except KeyError:
+                open_method, file_path = open, filename
+            with open_method(file_path, 'rt') as f:
                 command_output = f.read()
-            os.remove(filename)
+            os.remove(file_path)
         else:
             command_output = new_io.getvalue().strip()
         if format == "json":
@@ -87,10 +118,10 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         Site.objects.all().delete()
         # Load fixture 1. Single JSON file, with two objects.
         management.call_command('loaddata', 'fixture1.json', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Time to reform copyright>',
-            '<Article: Poker has no place on ESPN>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            ['Time to reform copyright', 'Poker has no place on ESPN'],
+        )
 
         # Dump the current contents of the database as a JSON fixture
         self._dumpdata_assert(
@@ -173,27 +204,33 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
 
         # Load fixture 2. JSON file imported by default. Overwrites some existing objects
         management.call_command('loaddata', 'fixture2.json', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Django conquers world!>',
-            '<Article: Copyright is fine the way it is>',
-            '<Article: Poker has no place on ESPN>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            [
+                'Django conquers world!',
+                'Copyright is fine the way it is',
+                'Poker has no place on ESPN',
+            ],
+        )
 
         # Load fixture 3, XML format.
         management.call_command('loaddata', 'fixture3.xml', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: XML identified as leading cause of cancer>',
-            '<Article: Django conquers world!>',
-            '<Article: Copyright is fine the way it is>',
-            '<Article: Poker on TV is great!>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            [
+                'XML identified as leading cause of cancer',
+                'Django conquers world!',
+                'Copyright is fine the way it is',
+                'Poker on TV is great!',
+            ],
+        )
 
         # Load fixture 6, JSON file with dynamic ContentType fields. Testing ManyToOne.
         management.call_command('loaddata', 'fixture6.json', verbosity=0)
         self.assertQuerysetEqual(Tag.objects.all(), [
             '<Tag: <Article: Copyright is fine the way it is> tagged "copyright">',
             '<Tag: <Article: Copyright is fine the way it is> tagged "law">',
-        ], ordered=False)
+        ], transform=repr, ordered=False)
 
         # Load fixture 7, XML file with dynamic ContentType fields. Testing ManyToOne.
         management.call_command('loaddata', 'fixture7.xml', verbosity=0)
@@ -202,7 +239,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             '<Tag: <Article: Copyright is fine the way it is> tagged "legal">',
             '<Tag: <Article: Django conquers world!> tagged "django">',
             '<Tag: <Article: Django conquers world!> tagged "world domination">',
-        ], ordered=False)
+        ], transform=repr, ordered=False)
 
         # Load fixture 8, JSON file with dynamic Permission fields. Testing ManyToMany.
         management.call_command('loaddata', 'fixture8.json', verbosity=0)
@@ -210,7 +247,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             '<Visa: Django Reinhardt Can add user, Can change user, Can delete user>',
             '<Visa: Stephane Grappelli Can add user>',
             '<Visa: Prince >'
-        ], ordered=False)
+        ], transform=repr, ordered=False)
 
         # Load fixture 9, XML file with dynamic Permission fields. Testing ManyToMany.
         management.call_command('loaddata', 'fixture9.xml', verbosity=0)
@@ -218,15 +255,18 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             '<Visa: Django Reinhardt Can add user, Can change user, Can delete user>',
             '<Visa: Stephane Grappelli Can add user, Can delete user>',
             '<Visa: Artist formerly known as "Prince" Can change user>'
-        ], ordered=False)
+        ], transform=repr, ordered=False)
 
         # object list is unaffected
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: XML identified as leading cause of cancer>',
-            '<Article: Django conquers world!>',
-            '<Article: Copyright is fine the way it is>',
-            '<Article: Poker on TV is great!>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            [
+                'XML identified as leading cause of cancer',
+                'Django conquers world!',
+                'Copyright is fine the way it is',
+                'Poker on TV is great!',
+            ],
+        )
 
         # By default, you get raw keys on dumpdata
         self._dumpdata_assert(
@@ -376,16 +416,18 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         with self.assertRaisesMessage(management.CommandError, "Unknown model: fixtures.FooModel"):
             self._dumpdata_assert(['fixtures', 'sites'], '', exclude_list=['fixtures.FooModel'])
 
-    @unittest.skipIf(sys.platform.startswith('win'), "Windows doesn't support '?' in filenames.")
+    @unittest.skipIf(sys.platform == 'win32', "Windows doesn't support '?' in filenames.")
     def test_load_fixture_with_special_characters(self):
         management.call_command('loaddata', 'fixture_with[special]chars', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), ['<Article: How To Deal With Special Characters>'])
+        self.assertEqual(
+            Article.objects.get().headline,
+            'How To Deal With Special Characters',
+        )
 
     def test_dumpdata_with_filtering_manager(self):
         spy1 = Spy.objects.create(name='Paul')
         spy2 = Spy.objects.create(name='Alex', cover_blown=True)
-        self.assertQuerysetEqual(Spy.objects.all(),
-                                 ['<Spy: Paul>'])
+        self.assertSequenceEqual(Spy.objects.all(), [spy1])
         # Use the default manager
         self._dumpdata_assert(
             ['fixtures.Spy'],
@@ -468,6 +510,66 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             filename='dumpdata.json'
         )
 
+    def test_dumpdata_with_file_gzip_output(self):
+        management.call_command('loaddata', 'fixture1.json', verbosity=0)
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"pk": 1, "model": "fixtures.category", "fields": {"description": "Latest news stories", "title": '
+            '"News Stories"}}, {"pk": 2, "model": "fixtures.article", "fields": {"headline": "Poker has no place '
+            'on ESPN", "pub_date": "2006-06-16T12:00:00"}}, {"pk": 3, "model": "fixtures.article", "fields": '
+            '{"headline": "Time to reform copyright", "pub_date": "2006-06-16T13:00:00"}}]',
+            filename='dumpdata.json.gz',
+        )
+
+    @unittest.skipUnless(HAS_BZ2, 'No bz2 library detected.')
+    def test_dumpdata_with_file_bz2_output(self):
+        management.call_command('loaddata', 'fixture1.json', verbosity=0)
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"pk": 1, "model": "fixtures.category", "fields": {"description": "Latest news stories", "title": '
+            '"News Stories"}}, {"pk": 2, "model": "fixtures.article", "fields": {"headline": "Poker has no place '
+            'on ESPN", "pub_date": "2006-06-16T12:00:00"}}, {"pk": 3, "model": "fixtures.article", "fields": '
+            '{"headline": "Time to reform copyright", "pub_date": "2006-06-16T13:00:00"}}]',
+            filename='dumpdata.json.bz2',
+        )
+
+    @unittest.skipUnless(HAS_LZMA, 'No lzma library detected.')
+    def test_dumpdata_with_file_lzma_output(self):
+        management.call_command('loaddata', 'fixture1.json', verbosity=0)
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"pk": 1, "model": "fixtures.category", "fields": {"description": "Latest news stories", "title": '
+            '"News Stories"}}, {"pk": 2, "model": "fixtures.article", "fields": {"headline": "Poker has no place '
+            'on ESPN", "pub_date": "2006-06-16T12:00:00"}}, {"pk": 3, "model": "fixtures.article", "fields": '
+            '{"headline": "Time to reform copyright", "pub_date": "2006-06-16T13:00:00"}}]',
+            filename='dumpdata.json.lzma',
+        )
+
+    @unittest.skipUnless(HAS_LZMA, 'No lzma library detected.')
+    def test_dumpdata_with_file_xz_output(self):
+        management.call_command('loaddata', 'fixture1.json', verbosity=0)
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"pk": 1, "model": "fixtures.category", "fields": {"description": "Latest news stories", "title": '
+            '"News Stories"}}, {"pk": 2, "model": "fixtures.article", "fields": {"headline": "Poker has no place '
+            'on ESPN", "pub_date": "2006-06-16T12:00:00"}}, {"pk": 3, "model": "fixtures.article", "fields": '
+            '{"headline": "Time to reform copyright", "pub_date": "2006-06-16T13:00:00"}}]',
+            filename='dumpdata.json.xz',
+        )
+
+    def test_dumpdata_with_file_zip_output(self):
+        management.call_command('loaddata', 'fixture1.json', verbosity=0)
+        msg = "Unsupported file extension (.zip). Fixtures saved in 'dumpdata.json'."
+        with self.assertWarnsMessage(RuntimeWarning, msg):
+            self._dumpdata_assert(
+                ['fixtures'],
+                '[{"pk": 1, "model": "fixtures.category", "fields": {"description": "Latest news stories", "title": '
+                '"News Stories"}}, {"pk": 2, "model": "fixtures.article", "fields": {"headline": "Poker has no place '
+                'on ESPN", "pub_date": "2006-06-16T12:00:00"}}, {"pk": 3, "model": "fixtures.article", "fields": '
+                '{"headline": "Time to reform copyright", "pub_date": "2006-06-16T13:00:00"}}]',
+                filename='dumpdata.json.zip',
+            )
+
     def test_dumpdata_progressbar(self):
         """
         Dumpdata shows a progress bar on the command line when --output is set,
@@ -522,38 +624,72 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
     def test_compress_format_loading(self):
         # Load fixture 4 (compressed), using format specification
         management.call_command('loaddata', 'fixture4.json', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Django pets kitten>',
-        ])
+        self.assertEqual(Article.objects.get().headline, 'Django pets kitten')
 
     def test_compressed_specified_loading(self):
         # Load fixture 5 (compressed), using format *and* compression specification
         management.call_command('loaddata', 'fixture5.json.zip', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: WoW subscribers now outnumber readers>',
-        ])
+        self.assertEqual(
+            Article.objects.get().headline,
+            'WoW subscribers now outnumber readers',
+        )
 
     def test_compressed_loading(self):
         # Load fixture 5 (compressed), only compression specification
         management.call_command('loaddata', 'fixture5.zip', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: WoW subscribers now outnumber readers>',
-        ])
+        self.assertEqual(
+            Article.objects.get().headline,
+            'WoW subscribers now outnumber readers',
+        )
+
+    def test_compressed_loading_gzip(self):
+        management.call_command('loaddata', 'fixture5.json.gz', verbosity=0)
+        self.assertEqual(
+            Article.objects.get().headline,
+            'WoW subscribers now outnumber readers',
+        )
+
+    @unittest.skipUnless(HAS_BZ2, 'No bz2 library detected.')
+    def test_compressed_loading_bz2(self):
+        management.call_command('loaddata', 'fixture5.json.bz2', verbosity=0)
+        self.assertEqual(
+            Article.objects.get().headline,
+            'WoW subscribers now outnumber readers',
+        )
+
+    @unittest.skipUnless(HAS_LZMA, 'No lzma library detected.')
+    def test_compressed_loading_lzma(self):
+        management.call_command('loaddata', 'fixture5.json.lzma', verbosity=0)
+        self.assertEqual(
+            Article.objects.get().headline,
+            'WoW subscribers now outnumber readers',
+        )
+
+    @unittest.skipUnless(HAS_LZMA, 'No lzma library detected.')
+    def test_compressed_loading_xz(self):
+        management.call_command('loaddata', 'fixture5.json.xz', verbosity=0)
+        self.assertEqual(
+            Article.objects.get().headline,
+            'WoW subscribers now outnumber readers',
+        )
 
     def test_ambiguous_compressed_fixture(self):
-        # The name "fixture5" is ambiguous, so loading it will raise an error
-        with self.assertRaises(management.CommandError) as cm:
+        # The name "fixture5" is ambiguous, so loading raises an error.
+        msg = "Multiple fixtures named 'fixture5'"
+        with self.assertRaisesMessage(management.CommandError, msg):
             management.call_command('loaddata', 'fixture5', verbosity=0)
-            self.assertIn("Multiple fixtures named 'fixture5'", cm.exception.args[0])
 
     def test_db_loading(self):
         # Load db fixtures 1 and 2. These will load using the 'default' database identifier implicitly
         management.call_command('loaddata', 'db_fixture_1', verbosity=0)
         management.call_command('loaddata', 'db_fixture_2', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Who needs more than one database?>',
-            '<Article: Who needs to use compressed data?>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            [
+                'Who needs more than one database?',
+                'Who needs to use compressed data?',
+            ],
+        )
 
     def test_loaddata_error_message(self):
         """
@@ -564,10 +700,11 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         # This won't affect other tests because the database connection
         # is closed at the end of each test.
         if connection.vendor == 'mysql':
-            connection.cursor().execute("SET sql_mode = 'TRADITIONAL'")
-        with self.assertRaises(IntegrityError) as cm:
+            with connection.cursor() as cursor:
+                cursor.execute("SET sql_mode = 'TRADITIONAL'")
+        msg = 'Could not load fixtures.Article(pk=1):'
+        with self.assertRaisesMessage(IntegrityError, msg):
             management.call_command('loaddata', 'invalid.json', verbosity=0)
-            self.assertIn("Could not load fixtures.Article(pk=1):", cm.exception.args[0])
 
     @unittest.skipUnless(connection.vendor == 'postgresql', 'psycopg2 prohibits null characters in data.')
     def test_loaddata_null_characters_on_postgresql(self):
@@ -583,9 +720,10 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             management.call_command('loaddata', 'db_fixture_1', verbosity=0, app_label="someotherapp")
         self.assertQuerysetEqual(Article.objects.all(), [])
         management.call_command('loaddata', 'db_fixture_1', verbosity=0, app_label="fixtures")
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Who needs more than one database?>',
-        ])
+        self.assertEqual(
+            Article.objects.get().headline,
+            'Who needs more than one database?',
+        )
 
     def test_loaddata_verbosity_three(self):
         output = StringIO()
@@ -601,10 +739,13 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         # Load db fixtures 1 and 2. These will load using the 'default' database identifier explicitly
         management.call_command('loaddata', 'db_fixture_1', verbosity=0, database='default')
         management.call_command('loaddata', 'db_fixture_2', verbosity=0, database='default')
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Who needs more than one database?>',
-            '<Article: Who needs to use compressed data?>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            [
+                'Who needs more than one database?',
+                'Who needs to use compressed data?',
+            ],
+        )
 
     def test_unmatched_identifier_loading(self):
         # Try to load db fixture 3. This won't load because the database identifier doesn't match
@@ -623,7 +764,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         self.assertQuerysetEqual(Tag.objects.all(), [
             '<Tag: <Article: Time to reform copyright> tagged "copyright">',
             '<Tag: <Article: Time to reform copyright> tagged "law">'
-        ], ordered=False)
+        ], transform=repr, ordered=False)
 
         # Dump the current contents of the database as a JSON fixture
         self._dumpdata_assert(
@@ -668,14 +809,14 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         management.call_command('loaddata', 'fixture1', exclude=['fixtures'], verbosity=0)
         self.assertFalse(Article.objects.exists())
         self.assertFalse(Category.objects.exists())
-        self.assertQuerysetEqual(Site.objects.all(), ['<Site: example.com>'])
+        self.assertEqual(Site.objects.get().domain, 'example.com')
 
     def test_loading_with_exclude_model(self):
         Site.objects.all().delete()
         management.call_command('loaddata', 'fixture1', exclude=['fixtures.Article'], verbosity=0)
         self.assertFalse(Article.objects.exists())
-        self.assertQuerysetEqual(Category.objects.all(), ['<Category: News Stories>'])
-        self.assertQuerysetEqual(Site.objects.all(), ['<Site: example.com>'])
+        self.assertEqual(Category.objects.get().title, 'News Stories')
+        self.assertEqual(Site.objects.get().domain, 'example.com')
 
     def test_exclude_option_errors(self):
         """Excluding a bogus app or model should raise an error."""
@@ -699,22 +840,23 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         fixture_json = os.path.join(tests_dir, 'fixtures', 'fixture1.json')
         fixture_xml = os.path.join(tests_dir, 'fixtures', 'fixture3.xml')
 
-        with mock.patch('django.core.management.commands.loaddata.sys.stdin', open(fixture_json, 'r')):
+        with mock.patch('django.core.management.commands.loaddata.sys.stdin', open(fixture_json)):
             management.call_command('loaddata', '--format=json', '-', verbosity=0)
-            self.assertEqual(Article.objects.count(), 2)
-            self.assertQuerysetEqual(Article.objects.all(), [
-                '<Article: Time to reform copyright>',
-                '<Article: Poker has no place on ESPN>',
-            ])
+            self.assertSequenceEqual(
+                Article.objects.values_list('headline', flat=True),
+                ['Time to reform copyright', 'Poker has no place on ESPN'],
+            )
 
-        with mock.patch('django.core.management.commands.loaddata.sys.stdin', open(fixture_xml, 'r')):
+        with mock.patch('django.core.management.commands.loaddata.sys.stdin', open(fixture_xml)):
             management.call_command('loaddata', '--format=xml', '-', verbosity=0)
-            self.assertEqual(Article.objects.count(), 3)
-            self.assertQuerysetEqual(Article.objects.all(), [
-                '<Article: XML identified as leading cause of cancer>',
-                '<Article: Time to reform copyright>',
-                '<Article: Poker on TV is great!>',
-            ])
+            self.assertSequenceEqual(
+                Article.objects.values_list('headline', flat=True),
+                [
+                    'XML identified as leading cause of cancer',
+                    'Time to reform copyright',
+                    'Poker on TV is great!',
+                ],
+            )
 
 
 class NonexistentFixtureTests(TestCase):
@@ -752,22 +894,22 @@ class FixtureTransactionTests(DumpDataAssertMixin, TransactionTestCase):
     def test_format_discovery(self):
         # Load fixture 1 again, using format discovery
         management.call_command('loaddata', 'fixture1', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Time to reform copyright>',
-            '<Article: Poker has no place on ESPN>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            ['Time to reform copyright', 'Poker has no place on ESPN'],
+        )
 
         # Try to load fixture 2 using format discovery; this will fail
         # because there are two fixture2's in the fixtures directory
-        with self.assertRaises(management.CommandError) as cm:
+        msg = "Multiple fixtures named 'fixture2'"
+        with self.assertRaisesMessage(management.CommandError, msg):
             management.call_command('loaddata', 'fixture2', verbosity=0)
-            self.assertIn("Multiple fixtures named 'fixture2'", cm.exception.args[0])
 
         # object list is unaffected
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Time to reform copyright>',
-            '<Article: Poker has no place on ESPN>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            ['Time to reform copyright', 'Poker has no place on ESPN'],
+        )
 
         # Dump the current contents of the database as a JSON fixture
         self._dumpdata_assert(
@@ -780,26 +922,123 @@ class FixtureTransactionTests(DumpDataAssertMixin, TransactionTestCase):
 
         # Load fixture 4 (compressed), using format discovery
         management.call_command('loaddata', 'fixture4', verbosity=0)
-        self.assertQuerysetEqual(Article.objects.all(), [
-            '<Article: Django pets kitten>',
-            '<Article: Time to reform copyright>',
-            '<Article: Poker has no place on ESPN>',
-        ])
+        self.assertSequenceEqual(
+            Article.objects.values_list('headline', flat=True),
+            [
+                'Django pets kitten',
+                'Time to reform copyright',
+                'Poker has no place on ESPN',
+            ],
+        )
 
 
-class ForwardReferenceTests(TestCase):
+class ForwardReferenceTests(DumpDataAssertMixin, TestCase):
     def test_forward_reference_fk(self):
         management.call_command('loaddata', 'forward_reference_fk.json', verbosity=0)
-        self.assertEqual(NaturalKeyThing.objects.count(), 2)
         t1, t2 = NaturalKeyThing.objects.all()
         self.assertEqual(t1.other_thing, t2)
         self.assertEqual(t2.other_thing, t1)
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"model": "fixtures.naturalkeything", "pk": 1, '
+            '"fields": {"key": "t1", "other_thing": 2, "other_things": []}}, '
+            '{"model": "fixtures.naturalkeything", "pk": 2, '
+            '"fields": {"key": "t2", "other_thing": 1, "other_things": []}}]',
+        )
+
+    def test_forward_reference_fk_natural_key(self):
+        management.call_command(
+            'loaddata',
+            'forward_reference_fk_natural_key.json',
+            verbosity=0,
+        )
+        t1, t2 = NaturalKeyThing.objects.all()
+        self.assertEqual(t1.other_thing, t2)
+        self.assertEqual(t2.other_thing, t1)
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"model": "fixtures.naturalkeything", '
+            '"fields": {"key": "t1", "other_thing": ["t2"], "other_things": []}}, '
+            '{"model": "fixtures.naturalkeything", '
+            '"fields": {"key": "t2", "other_thing": ["t1"], "other_things": []}}]',
+            natural_primary_keys=True,
+            natural_foreign_keys=True,
+        )
 
     def test_forward_reference_m2m(self):
         management.call_command('loaddata', 'forward_reference_m2m.json', verbosity=0)
         self.assertEqual(NaturalKeyThing.objects.count(), 3)
         t1 = NaturalKeyThing.objects.get_by_natural_key('t1')
-        self.assertQuerysetEqual(
-            t1.other_things.order_by('key'),
-            ['<NaturalKeyThing: t2>', '<NaturalKeyThing: t3>']
+        self.assertSequenceEqual(
+            t1.other_things.order_by('key').values_list('key', flat=True),
+            ['t2', 't3'],
+        )
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"model": "fixtures.naturalkeything", "pk": 1, '
+            '"fields": {"key": "t1", "other_thing": null, "other_things": [2, 3]}}, '
+            '{"model": "fixtures.naturalkeything", "pk": 2, '
+            '"fields": {"key": "t2", "other_thing": null, "other_things": []}}, '
+            '{"model": "fixtures.naturalkeything", "pk": 3, '
+            '"fields": {"key": "t3", "other_thing": null, "other_things": []}}]',
+        )
+
+    def test_forward_reference_m2m_natural_key(self):
+        management.call_command(
+            'loaddata',
+            'forward_reference_m2m_natural_key.json',
+            verbosity=0,
+        )
+        self.assertEqual(NaturalKeyThing.objects.count(), 3)
+        t1 = NaturalKeyThing.objects.get_by_natural_key('t1')
+        self.assertSequenceEqual(
+            t1.other_things.order_by('key').values_list('key', flat=True),
+            ['t2', 't3'],
+        )
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"model": "fixtures.naturalkeything", '
+            '"fields": {"key": "t1", "other_thing": null, "other_things": [["t2"], ["t3"]]}}, '
+            '{"model": "fixtures.naturalkeything", '
+            '"fields": {"key": "t2", "other_thing": null, "other_things": []}}, '
+            '{"model": "fixtures.naturalkeything", '
+            '"fields": {"key": "t3", "other_thing": null, "other_things": []}}]',
+            natural_primary_keys=True,
+            natural_foreign_keys=True,
+        )
+
+
+class CircularReferenceTests(DumpDataAssertMixin, TestCase):
+    def test_circular_reference(self):
+        management.call_command('loaddata', 'circular_reference.json', verbosity=0)
+        obj_a = CircularA.objects.get()
+        obj_b = CircularB.objects.get()
+        self.assertEqual(obj_a.obj, obj_b)
+        self.assertEqual(obj_b.obj, obj_a)
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"model": "fixtures.circulara", "pk": 1, '
+            '"fields": {"key": "x", "obj": 1}}, '
+            '{"model": "fixtures.circularb", "pk": 1, '
+            '"fields": {"key": "y", "obj": 1}}]',
+        )
+
+    def test_circular_reference_natural_key(self):
+        management.call_command(
+            'loaddata',
+            'circular_reference_natural_key.json',
+            verbosity=0,
+        )
+        obj_a = CircularA.objects.get()
+        obj_b = CircularB.objects.get()
+        self.assertEqual(obj_a.obj, obj_b)
+        self.assertEqual(obj_b.obj, obj_a)
+        self._dumpdata_assert(
+            ['fixtures'],
+            '[{"model": "fixtures.circulara", '
+            '"fields": {"key": "x", "obj": ["y"]}}, '
+            '{"model": "fixtures.circularb", '
+            '"fields": {"key": "y", "obj": ["x"]}}]',
+            natural_primary_keys=True,
+            natural_foreign_keys=True,
         )
