@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.core import management
 from django.core.management import CommandError
 from django.test import TestCase
@@ -8,11 +6,10 @@ from .models import Article
 
 
 class SampleTestCase(TestCase):
-    fixtures = ['fixture1.json', 'fixture2.json']
+    fixtures = ['model_package_fixture1.json', 'model_package_fixture2.json']
 
-    def testClassFixtures(self):
+    def test_class_fixtures(self):
         "Test cases can load fixture objects into models defined in packages"
-        self.assertEqual(Article.objects.count(), 3)
         self.assertQuerysetEqual(
             Article.objects.all(), [
                 "Django conquers world!",
@@ -28,7 +25,7 @@ class FixtureTestCase(TestCase):
     def test_loaddata(self):
         "Fixtures can load data into models defined in packages"
         # Load fixture 1. Single JSON file, with two objects
-        management.call_command("loaddata", "fixture1.json", verbosity=0)
+        management.call_command('loaddata', 'model_package_fixture1.json', verbosity=0)
         self.assertQuerysetEqual(
             Article.objects.all(), [
                 "Time to reform copyright",
@@ -39,7 +36,7 @@ class FixtureTestCase(TestCase):
 
         # Load fixture 2. JSON file imported by default. Overwrites some
         # existing objects
-        management.call_command("loaddata", "fixture2.json", verbosity=0)
+        management.call_command('loaddata', 'model_package_fixture2.json', verbosity=0)
         self.assertQuerysetEqual(
             Article.objects.all(), [
                 "Django conquers world!",
