@@ -1,6 +1,7 @@
 import datetime
 
-from django.forms import SplitDateTimeField, ValidationError
+from django.core.exceptions import ValidationError
+from django.forms import SplitDateTimeField
 from django.forms.widgets import SplitDateTimeWidget
 from django.test import SimpleTestCase
 
