@@ -1,6 +1,7 @@
 import uuid
 
-from django.forms import UUIDField, ValidationError
+from django.core.exceptions import ValidationError
+from django.forms import UUIDField
 from django.test import SimpleTestCase
 
 
@@ -18,8 +19,8 @@ class UUIDFieldTest(SimpleTestCase):
 
     def test_uuidfield_2(self):
         field = UUIDField(required=False)
-        value = field.clean('')
-        self.assertIsNone(value)
+        self.assertIsNone(field.clean(''))
+        self.assertIsNone(field.clean(None))
 
     def test_uuidfield_3(self):
         field = UUIDField()
