@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from django.forms import Widget
 from django.forms.widgets import Input
 
@@ -22,8 +19,8 @@ class WidgetTests(WidgetTest):
         self.assertIs(widget.value_omitted_from_data({'field': 'value'}, {}, 'field'), False)
 
     def test_no_trailing_newline_in_attrs(self):
-        self.check_html(Input(), 'name', 'value', strict=True, html='<input type="None" name="name" value="value" />')
+        self.check_html(Input(), 'name', 'value', strict=True, html='<input type="None" name="name" value="value">')
 
     def test_attr_false_not_rendered(self):
-        html = '<input type="None" name="name" value="value" />'
+        html = '<input type="None" name="name" value="value">'
         self.check_html(Input(), 'name', 'value', html=html, attrs={'readonly': False})
