@@ -1,7 +1,4 @@
-from __future__ import unicode_literals
-
 import time
-import unittest
 
 from django.core.exceptions import ImproperlyConfigured
 from django.http import HttpResponse
@@ -43,7 +40,7 @@ class DecoratedDispatchView(SimpleView):
 
     @decorator
     def dispatch(self, request, *args, **kwargs):
-        return super(DecoratedDispatchView, self).dispatch(request, *args, **kwargs)
+        return super().dispatch(request, *args, **kwargs)
 
 
 class AboutTemplateView(TemplateView):
@@ -67,7 +64,7 @@ class InstanceView(View):
         return self
 
 
-class ViewTest(unittest.TestCase):
+class ViewTest(SimpleTestCase):
     rf = RequestFactory()
 
     def _assert_simple(self, response):
@@ -78,14 +75,16 @@ class ViewTest(unittest.TestCase):
         """
         A view can't be accidentally instantiated before deployment
         """
-        with self.assertRaises(AttributeError):
+        msg = 'This method is available only on the class, not on instances.'
+        with self.assertRaisesMessage(AttributeError, msg):
             SimpleView(key='value').as_view()
 
     def test_no_init_args(self):
         """
         A view can't be accidentally instantiated before deployment
         """
-        with self.assertRaises(TypeError):
+        msg = 'as_view() takes 1 positional argument but 2 were given'
+        with self.assertRaisesMessage(TypeError, msg):
             SimpleView.as_view('value')
 
     def test_pathological_http_method(self):
@@ -114,6 +113,13 @@ class ViewTest(unittest.TestCase):
         response = SimpleView.as_view()(self.rf.head('/'))
         self.assertEqual(response.status_code, 200)
 
+    def test_setup_get_and_head(self):
+        view_instance = SimpleView()
+        self.assertFalse(hasattr(view_instance, 'head'))
+        view_instance.setup(self.rf.get('/'))
+        self.assertTrue(hasattr(view_instance, 'head'))
+        self.assertEqual(view_instance.head, view_instance.get)
+
     def test_head_no_get(self):
         """
         Test a view which supplies no GET method responds to HEAD with HTTP 405.
@@ -136,16 +142,24 @@ class ViewTest(unittest.TestCase):
         View arguments must be predefined on the class and can't
         be named like a HTTP method.
         """
+        msg = (
+            'The method name %s is not accepted as a keyword argument to '
+            'SimpleView().'
+        )
         # Check each of the allowed method names
         for method in SimpleView.http_method_names:
-            kwargs = dict(((method, "value"),))
-            with self.assertRaises(TypeError):
-                SimpleView.as_view(**kwargs)
+            with self.assertRaisesMessage(TypeError, msg % method):
+                SimpleView.as_view(**{method: 'value'})
 
         # Check the case view argument is ok if predefined on the class...
         CustomizableView.as_view(parameter="value")
         # ...but raises errors otherwise.
-        with self.assertRaises(TypeError):
+        msg = (
+            "CustomizableView() received an invalid keyword 'foobar'. "
+            "as_view only accepts arguments that are already attributes of "
+            "the class."
+        )
+        with self.assertRaisesMessage(TypeError, msg):
             CustomizableView.as_view(foobar="value")
 
     def test_calling_more_than_once(self):
@@ -181,7 +195,7 @@ class ViewTest(unittest.TestCase):
         view = SimpleView.as_view()
         response = view(request)
         self.assertEqual(200, response.status_code)
-        self.assertTrue(response['Allow'])
+        self.assertTrue(response.headers['Allow'])
 
     def test_options_for_get_view(self):
         """
@@ -212,7 +226,7 @@ class ViewTest(unittest.TestCase):
 
     def _assert_allows(self, response, *expected_methods):
         "Assert allowed HTTP methods reported in the Allow response header"
-        response_allows = set(response['Allow'].split(', '))
+        response_allows = set(response.headers['Allow'].split(', '))
         self.assertEqual(set(expected_methods + ('OPTIONS',)), response_allows)
 
     def test_args_kwargs_request_on_self(self):
@@ -226,6 +240,43 @@ class ViewTest(unittest.TestCase):
             self.assertNotIn(attribute, dir(bare_view))
             self.assertIn(attribute, dir(view))
 
+    def test_overridden_setup(self):
+        class SetAttributeMixin:
+            def setup(self, request, *args, **kwargs):
+                self.attr = True
+                super().setup(request, *args, **kwargs)
+
+        class CheckSetupView(SetAttributeMixin, SimpleView):
+            def dispatch(self, request, *args, **kwargs):
+                assert hasattr(self, 'attr')
+                return super().dispatch(request, *args, **kwargs)
+
+        response = CheckSetupView.as_view()(self.rf.get('/'))
+        self.assertEqual(response.status_code, 200)
+
+    def test_not_calling_parent_setup_error(self):
+        class TestView(View):
+            def setup(self, request, *args, **kwargs):
+                pass  # Not calling super().setup()
+
+        msg = (
+            "TestView instance has no 'request' attribute. Did you override "
+            "setup() and forget to call super()?"
+        )
+        with self.assertRaisesMessage(AttributeError, msg):
+            TestView.as_view()(self.rf.get('/'))
+
+    def test_setup_adds_args_kwargs_request(self):
+        request = self.rf.get('/')
+        args = ('arg 1', 'arg 2')
+        kwargs = {'kwarg_1': 1, 'kwarg_2': 'year'}
+
+        view = View()
+        view.setup(request, *args, **kwargs)
+        self.assertEqual(request, view.request)
+        self.assertEqual(args, view.args)
+        self.assertEqual(kwargs, view.kwargs)
+
     def test_direct_instantiation(self):
         """
         It should be possible to use the view by directly instantiating it
@@ -276,7 +327,11 @@ class TemplateViewTest(SimpleTestCase):
         """
         A template view must provide a template name.
         """
-        with self.assertRaises(ImproperlyConfigured):
+        msg = (
+            "TemplateResponseMixin requires either a definition of "
+            "'template_name' or an implementation of 'get_template_names()'"
+        )
+        with self.assertRaisesMessage(ImproperlyConfigured, msg):
             self.client.get('/template/no_template/')
 
     @require_jinja2
@@ -335,7 +390,7 @@ class TemplateViewTest(SimpleTestCase):
 
     def test_content_type(self):
         response = self.client.get('/template/content_type/')
-        self.assertEqual(response['Content-Type'], 'text/plain')
+        self.assertEqual(response.headers['Content-Type'], 'text/plain')
 
     def test_resolve_view(self):
         match = resolve('/template/content_type/')
@@ -346,6 +401,10 @@ class TemplateViewTest(SimpleTestCase):
         match = resolve('/template/login_required/')
         self.assertIs(match.func.view_class, TemplateView)
 
+    def test_extra_context(self):
+        response = self.client.get('/template/extra_context/')
+        self.assertEqual(response.context['title'], 'Title')
+
 
 @override_settings(ROOT_URLCONF='generic_views.urls')
 class RedirectViewTest(SimpleTestCase):
@@ -402,17 +461,12 @@ class RedirectViewTest(SimpleTestCase):
         "Named pattern parameter should reverse to the matching pattern"
         response = RedirectView.as_view(pattern_name='artist_detail')(self.rf.get('/foo/'), pk=1)
         self.assertEqual(response.status_code, 302)
-        self.assertEqual(response['Location'], '/detail/artist/1/')
+        self.assertEqual(response.headers['Location'], '/detail/artist/1/')
 
     def test_named_url_pattern_using_args(self):
         response = RedirectView.as_view(pattern_name='artist_detail')(self.rf.get('/foo/'), 1)
         self.assertEqual(response.status_code, 302)
-        self.assertEqual(response['Location'], '/detail/artist/1/')
-
-    def test_wrong_named_url_pattern(self):
-        "A wrong pattern name returns 410 GONE"
-        response = RedirectView.as_view(pattern_name='wrong.pattern_name')(self.rf.get('/foo/'))
-        self.assertEqual(response.status_code, 410)
+        self.assertEqual(response.headers['Location'], '/detail/artist/1/')
 
     def test_redirect_POST(self):
         "Default is a temporary redirect"
@@ -466,7 +520,7 @@ class RedirectViewTest(SimpleTestCase):
         self.assertEqual(response.status_code, 410)
 
 
-class GetContextDataTest(unittest.TestCase):
+class GetContextDataTest(SimpleTestCase):
 
     def test_get_context_data_super(self):
         test_view = views.CustomContextView()
@@ -495,7 +549,7 @@ class GetContextDataTest(unittest.TestCase):
         self.assertEqual(context['object'], test_view.object)
 
 
-class UseMultipleObjectMixinTest(unittest.TestCase):
+class UseMultipleObjectMixinTest(SimpleTestCase):
     rf = RequestFactory()
 
     def test_use_queryset_from_view(self):
@@ -515,7 +569,7 @@ class UseMultipleObjectMixinTest(unittest.TestCase):
         self.assertEqual(context['object_list'], queryset)
 
 
-class SingleObjectTemplateResponseMixinTest(unittest.TestCase):
+class SingleObjectTemplateResponseMixinTest(SimpleTestCase):
 
     def test_template_mixin_without_template(self):
         """
@@ -524,5 +578,9 @@ class SingleObjectTemplateResponseMixinTest(unittest.TestCase):
         TemplateDoesNotExist.
         """
         view = views.TemplateResponseWithoutTemplate()
-        with self.assertRaises(ImproperlyConfigured):
+        msg = (
+            "TemplateResponseMixin requires either a definition of "
+            "'template_name' or an implementation of 'get_template_names()'"
+        )
+        with self.assertRaisesMessage(ImproperlyConfigured, msg):
             view.get_template_names()
