@@ -187,6 +187,14 @@ urlpatterns = [
         'dates/books/<int:year>/week/<int:week>/unknown_week_format/',
         views.BookWeekArchive.as_view(week_format='%T'),
     ),
+    path(
+        'dates/books/<int:year>/week/<int:week>/iso_format/',
+        views.BookWeekArchive.as_view(year_format='%G', week_format='%V'),
+    ),
+    path(
+        'dates/books/<int:year>/week/<int:week>/invalid_iso_week_year_format/',
+        views.BookWeekArchive.as_view(week_format='%V'),
+    ),
     path('dates/booksignings/<int:year>/week/<int:week>/', views.BookSigningWeekArchive.as_view()),
 
     # DayArchiveView
