@@ -19,3 +19,6 @@ class Fields3D(models.Model):
     pointg = models.PointField(dim=3, geography=True)
     line = models.LineStringField(dim=3)
     poly = models.PolygonField(dim=3)
+
+    class Meta:
+        required_db_features = {'supports_3d_storage'}
