@@ -1,4 +1,4 @@
-from ..models import models
+from django.contrib.gis.db import models
 
 
 class RasterModel(models.Model):
