@@ -1,6 +1,7 @@
+import asyncio
 from http import HTTPStatus
 
-from django.core.exceptions import SuspiciousOperation
+from django.core.exceptions import BadRequest, SuspiciousOperation
 from django.db import connection, transaction
 from django.http import HttpResponse, StreamingHttpResponse
 from django.views.decorators.csrf import csrf_exempt
@@ -32,6 +33,10 @@ def not_in_transaction(request):
     return HttpResponse(str(connection.in_atomic_block))
 
 
+def bad_request(request):
+    raise BadRequest()
+
+
 def suspicious(request):
     raise SuspiciousOperation('dubious')
 
@@ -44,3 +49,21 @@ def malformed_post(request):
 
 def httpstatus_enum(request):
     return HttpResponse(status=HTTPStatus.OK)
+
+
+async def async_regular(request):
+    return HttpResponse(b'regular content')
+
+
+class CoroutineClearingView:
+    def __call__(self, request):
+        """Return an unawaited coroutine (common error for async views)."""
+        # Store coroutine to suppress 'unawaited' warning message
+        self._unawaited_coroutine = asyncio.sleep(0)
+        return self._unawaited_coroutine
+
+    def __del__(self):
+        self._unawaited_coroutine.close()
+
+
+async_unawaited = CoroutineClearingView()
