@@ -20,6 +20,6 @@ class ContentTypeTests(TestCase):
     def test_verbose_name(self):
         company_type = ContentType.objects.get(app_label='i18n', model='company')
         with translation.override('en'):
-            self.assertEqual(str(company_type), 'Company')
+            self.assertEqual(str(company_type), 'i18n | Company')
         with translation.override('fr'):
-            self.assertEqual(str(company_type), 'Société')
+            self.assertEqual(str(company_type), 'i18n | Société')
