@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
 class School(models.Model):
@@ -18,7 +16,6 @@ class Child(models.Model):
     name = models.CharField(max_length=100)
 
 
-@python_2_unicode_compatible
 class Poet(models.Model):
     name = models.CharField(max_length=100)
 
@@ -26,7 +23,6 @@ class Poet(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Poem(models.Model):
     poet = models.ForeignKey(Poet, models.CASCADE)
     name = models.CharField(max_length=100)
