@@ -67,6 +67,11 @@ class Season(models.Model):
     gt = models.IntegerField(null=True, blank=True)
     nulled_text_field = NulledTextField(null=True)
 
+    class Meta:
+        constraints = [
+            models.UniqueConstraint(fields=['year'], name='season_year_unique'),
+        ]
+
     def __str__(self):
         return str(self.year)
 
@@ -76,17 +81,11 @@ class Game(models.Model):
     home = models.CharField(max_length=100)
     away = models.CharField(max_length=100)
 
-    def __str__(self):
-        return "%s at %s" % (self.away, self.home)
-
 
 class Player(models.Model):
     name = models.CharField(max_length=100)
     games = models.ManyToManyField(Game, related_name='players')
 
-    def __str__(self):
-        return self.name
-
 
 class Product(models.Model):
     name = models.CharField(max_length=80)
@@ -96,3 +95,15 @@ class Product(models.Model):
 class Stock(models.Model):
     product = models.ForeignKey(Product, models.CASCADE)
     qty_available = models.DecimalField(max_digits=6, decimal_places=2)
+
+
+class Freebie(models.Model):
+    gift_product = models.ForeignKey(Product, models.CASCADE)
+    stock_id = models.IntegerField(blank=True, null=True)
+
+    stock = models.ForeignObject(
+        Stock,
+        from_fields=['stock_id', 'gift_product'],
+        to_fields=['id', 'product'],
+        on_delete=models.CASCADE,
+    )
