@@ -6,7 +6,7 @@ from .models import Article, Reporter, Writer
 
 
 class M2MIntermediaryTests(TestCase):
-    def test_intermeiary(self):
+    def test_intermediary(self):
         r1 = Reporter.objects.create(first_name="John", last_name="Smith")
         r2 = Reporter.objects.create(first_name="Jane", last_name="Doe")
 
