@@ -7,9 +7,6 @@ class Part(models.Model):
     class Meta:
         ordering = ('name',)
 
-    def __str__(self):
-        return self.name
-
 
 class Car(models.Model):
     name = models.CharField(max_length=20)
@@ -19,9 +16,6 @@ class Car(models.Model):
     class Meta:
         ordering = ('name',)
 
-    def __str__(self):
-        return self.name
-
 
 class SportsCar(Car):
     price = models.IntegerField()
@@ -34,6 +28,3 @@ class Person(models.Model):
 
     class Meta:
         ordering = ('name',)
-
-    def __str__(self):
-        return self.name
