@@ -3,30 +3,26 @@ Tests for the order_with_respect_to Meta attribute.
 """
 
 from django.db import models
-from django.utils import six
-from django.utils.encoding import python_2_unicode_compatible
 
 
 class Question(models.Model):
     text = models.CharField(max_length=200)
 
 
-@python_2_unicode_compatible
 class Answer(models.Model):
     text = models.CharField(max_length=200)
-    question = models.ForeignKey(Question)
+    question = models.ForeignKey(Question, models.CASCADE)
 
     class Meta:
         order_with_respect_to = 'question'
 
     def __str__(self):
-        return six.text_type(self.text)
+        return self.text
 
 
-@python_2_unicode_compatible
 class Post(models.Model):
     title = models.CharField(max_length=200)
-    parent = models.ForeignKey("self", related_name="children", null=True)
+    parent = models.ForeignKey("self", models.SET_NULL, related_name="children", null=True)
 
     class Meta:
         order_with_respect_to = "parent"
@@ -41,11 +37,11 @@ class Entity(models.Model):
 
 
 class Dimension(models.Model):
-    entity = models.OneToOneField('Entity', primary_key=True)
+    entity = models.OneToOneField('Entity', primary_key=True, on_delete=models.CASCADE)
 
 
 class Component(models.Model):
-    dimension = models.ForeignKey('Dimension')
+    dimension = models.ForeignKey('Dimension', on_delete=models.CASCADE)
 
     class Meta:
         order_with_respect_to = 'dimension'
