@@ -1,12 +1,13 @@
 from datetime import datetime
 from operator import attrgetter
 
-from django.db.models import Count, DateTimeField, F, Max, OuterRef, Subquery
+from django.db.models import (
+    CharField, DateTimeField, F, Max, OuterRef, Subquery, Value,
+)
 from django.db.models.functions import Upper
 from django.test import TestCase
-from django.utils.deprecation import RemovedInDjango31Warning
 
-from .models import Article, Author, OrderedByFArticle, Reference
+from .models import Article, Author, ChildArticle, OrderedByFArticle, Reference
 
 
 class OrderingTests(TestCase):
@@ -210,6 +211,15 @@ class OrderingTests(TestCase):
     def test_reverse_ordering_pure(self):
         qs1 = Article.objects.order_by(F('headline').asc())
         qs2 = qs1.reverse()
+        self.assertQuerysetEqual(
+            qs2, [
+                'Article 4',
+                'Article 3',
+                'Article 2',
+                'Article 1',
+            ],
+            attrgetter('headline'),
+        )
         self.assertQuerysetEqual(
             qs1, [
                 "Article 1",
@@ -219,14 +229,29 @@ class OrderingTests(TestCase):
             ],
             attrgetter("headline")
         )
+
+    def test_reverse_meta_ordering_pure(self):
+        Article.objects.create(
+            headline='Article 5',
+            pub_date=datetime(2005, 7, 30),
+            author=self.author_1,
+            second_author=self.author_2,
+        )
+        Article.objects.create(
+            headline='Article 5',
+            pub_date=datetime(2005, 7, 30),
+            author=self.author_2,
+            second_author=self.author_1,
+        )
         self.assertQuerysetEqual(
-            qs2, [
-                "Article 4",
-                "Article 3",
-                "Article 2",
-                "Article 1",
-            ],
-            attrgetter("headline")
+            Article.objects.filter(headline='Article 5').reverse(),
+            ['Name 2', 'Name 1'],
+            attrgetter('author.name'),
+        )
+        self.assertQuerysetEqual(
+            Article.objects.filter(headline='Article 5'),
+            ['Name 1', 'Name 2'],
+            attrgetter('author.name'),
         )
 
     def test_no_reordering_after_slicing(self):
@@ -290,10 +315,9 @@ class OrderingTests(TestCase):
         """
         'pk' works as an ordering option in Meta.
         """
-        self.assertQuerysetEqual(
-            Author.objects.all(),
-            list(reversed(range(1, Author.objects.count() + 1))),
-            attrgetter("pk"),
+        self.assertEqual(
+            [a.pk for a in Author.objects.all()],
+            [a.pk for a in Author.objects.order_by('-pk')],
         )
 
     def test_order_by_fk_attname(self):
@@ -301,8 +325,9 @@ class OrderingTests(TestCase):
         ordering by a foreign key by its attribute name prevents the query
         from inheriting its related model ordering option (#19195).
         """
+        authors = list(Author.objects.order_by('id'))
         for i in range(1, 5):
-            author = Author.objects.get(pk=i)
+            author = authors[i - 1]
             article = getattr(self, "a%d" % (5 - i))
             article.author = author
             article.save(update_fields={'author'})
@@ -317,6 +342,22 @@ class OrderingTests(TestCase):
             attrgetter("headline")
         )
 
+    def test_order_by_self_referential_fk(self):
+        self.a1.author = Author.objects.create(editor=self.author_1)
+        self.a1.save()
+        self.a2.author = Author.objects.create(editor=self.author_2)
+        self.a2.save()
+        self.assertQuerysetEqual(
+            Article.objects.filter(author__isnull=False).order_by('author__editor'),
+            ['Article 2', 'Article 1'],
+            attrgetter('headline'),
+        )
+        self.assertQuerysetEqual(
+            Article.objects.filter(author__isnull=False).order_by('author__editor_id'),
+            ['Article 1', 'Article 2'],
+            attrgetter('headline'),
+        )
+
     def test_order_by_f_expression(self):
         self.assertQuerysetEqual(
             Article.objects.order_by(F('headline')), [
@@ -378,6 +419,25 @@ class OrderingTests(TestCase):
             attrgetter("headline")
         )
 
+    def test_order_by_constant_value(self):
+        # Order by annotated constant from selected columns.
+        qs = Article.objects.annotate(
+            constant=Value('1', output_field=CharField()),
+        ).order_by('constant', '-headline')
+        self.assertSequenceEqual(qs, [self.a4, self.a3, self.a2, self.a1])
+        # Order by annotated constant which is out of selected columns.
+        self.assertSequenceEqual(
+            qs.values_list('headline', flat=True), [
+                'Article 4',
+                'Article 3',
+                'Article 2',
+                'Article 1',
+            ],
+        )
+        # Order by constant.
+        qs = Article.objects.order_by(Value('1', output_field=CharField()), '-headline')
+        self.assertSequenceEqual(qs, [self.a4, self.a3, self.a2, self.a1])
+
     def test_related_ordering_duplicate_table_reference(self):
         """
         An ordering referencing a model with an ordering referencing a model
@@ -405,12 +465,22 @@ class OrderingTests(TestCase):
             attrgetter('headline')
         )
 
-    def test_deprecated_values_annotate(self):
-        msg = (
-            "Article QuerySet won't use Meta.ordering in Django 3.1. Add "
-            ".order_by('-pub_date', 'headline', OrderBy(F(author__name), "
-            "descending=False), OrderBy(F(second_author__name), "
-            "descending=False)) to retain the current query."
+    def test_order_by_ptr_field_with_default_ordering_by_expression(self):
+        ca1 = ChildArticle.objects.create(
+            headline='h2',
+            pub_date=datetime(2005, 7, 27),
+            author=self.author_2,
         )
-        with self.assertRaisesMessage(RemovedInDjango31Warning, msg):
-            list(Article.objects.values('author').annotate(Count('headline')))
+        ca2 = ChildArticle.objects.create(
+            headline='h2',
+            pub_date=datetime(2005, 7, 27),
+            author=self.author_1,
+        )
+        ca3 = ChildArticle.objects.create(
+            headline='h3',
+            pub_date=datetime(2005, 7, 27),
+            author=self.author_1,
+        )
+        ca4 = ChildArticle.objects.create(headline='h1', pub_date=datetime(2005, 7, 28))
+        articles = ChildArticle.objects.order_by('article_ptr')
+        self.assertSequenceEqual(articles, [ca4, ca2, ca1, ca3])
