@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.contrib import admin
 from django.contrib.auth.models import User as AuthUser
 from django.contrib.contenttypes.models import ContentType
@@ -278,6 +276,13 @@ class ProxyModelTests(TestCase):
         resp = [u.name for u in UserProxy.objects.all()]
         self.assertEqual(resp, ['Bruce'])
 
+    def test_proxy_update(self):
+        user = User.objects.create(name='Bruce')
+        with self.assertNumQueries(1):
+            UserProxy.objects.filter(id=user.id).update(name='George')
+        user.refresh_from_db()
+        self.assertEqual(user.name, 'George')
+
     def test_select_related(self):
         """
         We can still use `select_related()` to include related models in our
