@@ -2,11 +2,8 @@ import unittest
 
 from django.db import NotSupportedError, connection, transaction
 from django.db.models import Count
-from django.test import (
-    TestCase, ignore_warnings, skipIfDBFeature, skipUnlessDBFeature,
-)
+from django.test import TestCase, skipIfDBFeature, skipUnlessDBFeature
 from django.test.utils import CaptureQueriesContext
-from django.utils.deprecation import RemovedInDjango31Warning
 
 from .models import Tag
 
@@ -14,7 +11,6 @@ from .models import Tag
 @skipUnlessDBFeature('supports_explaining_query_execution')
 class ExplainTests(TestCase):
 
-    @ignore_warnings(category=RemovedInDjango31Warning)
     def test_basic(self):
         querysets = [
             Tag.objects.filter(name='test'),
@@ -30,9 +26,6 @@ class ExplainTests(TestCase):
         for idx, queryset in enumerate(querysets):
             for format in all_formats:
                 with self.subTest(format=format, queryset=idx):
-                    if connection.vendor == 'mysql':
-                        # This does a query and caches the result.
-                        connection.features.needs_explain_extended
                     with self.assertNumQueries(1), CaptureQueriesContext(connection) as captured_queries:
                         result = queryset.explain(format=format)
                         self.assertTrue(captured_queries[0]['sql'].startswith(connection.ops.explain_prefix))
@@ -41,8 +34,8 @@ class ExplainTests(TestCase):
 
     @skipUnlessDBFeature('validates_explain_options')
     def test_unknown_options(self):
-        with self.assertRaisesMessage(ValueError, "Unknown options: TEST, TEST2"):
-            Tag.objects.all().explain(**{"TEST": 1, "TEST2": 1})
+        with self.assertRaisesMessage(ValueError, 'Unknown options: test, test2'):
+            Tag.objects.all().explain(test=1, test2=1)
 
     def test_unknown_format(self):
         msg = 'DOES NOT EXIST is not a recognized format.'
@@ -60,8 +53,12 @@ class ExplainTests(TestCase):
             {'verbose': True, 'timing': True, 'analyze': True},
             {'verbose': False, 'timing': False, 'analyze': True},
         ]
-        if connection.pg_version >= 100000:
+        if connection.features.is_postgresql_10:
             test_options.append({'summary': True})
+        if connection.features.is_postgresql_12:
+            test_options.append({'settings': True})
+        if connection.features.is_postgresql_13:
+            test_options.append({'analyze': True, 'wal': True})
         for options in test_options:
             with self.subTest(**options), transaction.atomic():
                 with CaptureQueriesContext(connection) as captured_queries:
@@ -71,59 +68,31 @@ class ExplainTests(TestCase):
                     option = '{} {}'.format(name.upper(), 'true' if value else 'false')
                     self.assertIn(option, captured_queries[0]['sql'])
 
-    def test_option_sql_injection(self):
-        qs = Tag.objects.filter(name="test")
-        options = {"SUMMARY true) SELECT 1; --": True}
-        msg = "Invalid option name: 'SUMMARY true) SELECT 1; --'"
-        with self.assertRaisesMessage(ValueError, msg):
-            qs.explain(**options)
-
-    def test_invalid_option_names(self):
-        qs = Tag.objects.filter(name="test")
-        tests = [
-            'opt"ion',
-            "o'ption",
-            "op`tion",
-            "opti on",
-            "option--",
-            "optio\tn",
-            "o\nption",
-            "option;",
-            "你 好",
-            # [] are used by MSSQL.
-            "option[",
-            "option]",
-        ]
-        for invalid_option in tests:
-            with self.subTest(invalid_option):
-                msg = "Invalid option name: '%s'" % invalid_option
-                with self.assertRaisesMessage(ValueError, msg):
-                    qs.explain(**{invalid_option: True})
-
     @unittest.skipUnless(connection.vendor == 'mysql', 'MySQL specific')
     def test_mysql_text_to_traditional(self):
-        # Initialize the cached property, if needed, to prevent a query for
-        # the MySQL version during the QuerySet evaluation.
-        connection.features.needs_explain_extended
+        # Ensure these cached properties are initialized to prevent queries for
+        # the MariaDB or MySQL version during the QuerySet evaluation.
+        connection.features.supported_explain_formats
         with CaptureQueriesContext(connection) as captured_queries:
             Tag.objects.filter(name='test').explain(format='text')
         self.assertEqual(len(captured_queries), 1)
         self.assertIn('FORMAT=TRADITIONAL', captured_queries[0]['sql'])
 
-    @unittest.skipUnless(connection.vendor == 'mysql', 'MySQL < 5.7 specific')
-    def test_mysql_extended(self):
-        # Inner skip to avoid module level query for MySQL version.
-        if not connection.features.needs_explain_extended:
-            raise unittest.SkipTest('MySQL < 5.7 specific')
+    @unittest.skipUnless(connection.vendor == 'mysql', 'MariaDB and MySQL >= 8.0.18 specific.')
+    def test_mysql_analyze(self):
         qs = Tag.objects.filter(name='test')
         with CaptureQueriesContext(connection) as captured_queries:
-            qs.explain(format='json')
+            qs.explain(analyze=True)
         self.assertEqual(len(captured_queries), 1)
-        self.assertNotIn('EXTENDED', captured_queries[0]['sql'])
+        prefix = 'ANALYZE ' if connection.mysql_is_mariadb else 'EXPLAIN ANALYZE '
+        self.assertTrue(captured_queries[0]['sql'].startswith(prefix))
         with CaptureQueriesContext(connection) as captured_queries:
-            qs.explain(format='text')
+            qs.explain(analyze=True, format='JSON')
         self.assertEqual(len(captured_queries), 1)
-        self.assertNotIn('EXTENDED', captured_queries[0]['sql'])
+        if connection.mysql_is_mariadb:
+            self.assertIn('FORMAT=JSON', captured_queries[0]['sql'])
+        else:
+            self.assertNotIn('FORMAT=JSON', captured_queries[0]['sql'])
 
 
 @skipIfDBFeature('supports_explaining_query_execution')
