@@ -1,4 +1,3 @@
-from collections import OrderedDict
 from datetime import date, datetime
 
 from django.conf.urls.i18n import i18n_patterns
@@ -35,6 +34,18 @@ class SimpleI18nSitemap(Sitemap):
         return I18nTestModel.objects.order_by('pk').all()
 
 
+class AlternatesI18nSitemap(SimpleI18nSitemap):
+    alternates = True
+
+
+class LimitedI18nSitemap(AlternatesI18nSitemap):
+    languages = ['en', 'es']
+
+
+class XDefaultI18nSitemap(AlternatesI18nSitemap):
+    x_default = True
+
+
 class EmptySitemap(Sitemap):
     changefreq = "never"
     priority = 0.5
@@ -78,8 +89,20 @@ simple_sitemaps = {
     'simple': SimpleSitemap,
 }
 
-simple_i18nsitemaps = {
-    'simple': SimpleI18nSitemap,
+simple_i18n_sitemaps = {
+    'i18n': SimpleI18nSitemap,
+}
+
+alternates_i18n_sitemaps = {
+    'i18n-alternates': AlternatesI18nSitemap,
+}
+
+limited_i18n_sitemaps = {
+    'i18n-limited': LimitedI18nSitemap,
+}
+
+xdefault_i18n_sitemaps = {
+    'i18n-xdefault': XDefaultI18nSitemap,
 }
 
 simple_sitemaps_not_callable = {
@@ -98,31 +121,31 @@ fixed_lastmod_sitemaps = {
     'fixed-lastmod': FixedLastmodSitemap,
 }
 
-fixed_lastmod__mixed_sitemaps = {
+fixed_lastmod_mixed_sitemaps = {
     'fixed-lastmod-mixed': FixedLastmodMixedSitemap,
 }
 
-sitemaps_lastmod_mixed_ascending = OrderedDict([
-    ('no-lastmod', EmptySitemap),
-    ('lastmod', FixedLastmodSitemap),
-])
+sitemaps_lastmod_mixed_ascending = {
+    'no-lastmod': EmptySitemap,
+    'lastmod': FixedLastmodSitemap,
+}
 
-sitemaps_lastmod_mixed_descending = OrderedDict([
-    ('lastmod', FixedLastmodSitemap),
-    ('no-lastmod', EmptySitemap),
-])
+sitemaps_lastmod_mixed_descending = {
+    'lastmod': FixedLastmodSitemap,
+    'no-lastmod': EmptySitemap,
+}
 
-sitemaps_lastmod_ascending = OrderedDict([
-    ('date', DateSiteMap),
-    ('datetime', FixedLastmodSitemap),
-    ('datetime-newer', FixedNewerLastmodSitemap),
-])
+sitemaps_lastmod_ascending = {
+    'date': DateSiteMap,
+    'datetime': FixedLastmodSitemap,
+    'datetime-newer': FixedNewerLastmodSitemap,
+}
 
-sitemaps_lastmod_descending = OrderedDict([
-    ('datetime-newer', FixedNewerLastmodSitemap),
-    ('datetime', FixedLastmodSitemap),
-    ('date', DateSiteMap),
-])
+sitemaps_lastmod_descending = {
+    'datetime-newer': FixedNewerLastmodSitemap,
+    'datetime': FixedLastmodSitemap,
+    'date': DateSiteMap,
+}
 
 generic_sitemaps = {
     'generic': GenericSitemap({'queryset': TestModel.objects.order_by('pk').all()}),
@@ -152,7 +175,19 @@ urlpatterns = [
         name='django.contrib.sitemaps.views.sitemap'),
     path(
         'simple/i18n.xml', views.sitemap,
-        {'sitemaps': simple_i18nsitemaps},
+        {'sitemaps': simple_i18n_sitemaps},
+        name='django.contrib.sitemaps.views.sitemap'),
+    path(
+        'alternates/i18n.xml', views.sitemap,
+        {'sitemaps': alternates_i18n_sitemaps},
+        name='django.contrib.sitemaps.views.sitemap'),
+    path(
+        'limited/i18n.xml', views.sitemap,
+        {'sitemaps': limited_i18n_sitemaps},
+        name='django.contrib.sitemaps.views.sitemap'),
+    path(
+        'x-default/i18n.xml', views.sitemap,
+        {'sitemaps': xdefault_i18n_sitemaps},
         name='django.contrib.sitemaps.views.sitemap'),
     path(
         'simple/custom-sitemap.xml', views.sitemap,
@@ -168,7 +203,7 @@ urlpatterns = [
         name='django.contrib.sitemaps.views.sitemap'),
     path(
         'lastmod-mixed/sitemap.xml', views.sitemap,
-        {'sitemaps': fixed_lastmod__mixed_sitemaps},
+        {'sitemaps': fixed_lastmod_mixed_sitemaps},
         name='django.contrib.sitemaps.views.sitemap'),
     path(
         'lastmod/date-sitemap.xml', views.sitemap,
