@@ -1,5 +1,5 @@
-from django.conf.urls import url
 from django.contrib.sitemaps import views
+from django.urls import path
 
 from .http import SimpleSitemap
 
@@ -13,8 +13,9 @@ secure_sitemaps = {
 }
 
 urlpatterns = [
-    url(r'^secure/index\.xml$', views.index, {'sitemaps': secure_sitemaps}),
-    url(r'^secure/sitemap-(?P<section>.+)\.xml$', views.sitemap,
+    path('secure/index.xml', views.index, {'sitemaps': secure_sitemaps}),
+    path(
+        'secure/sitemap-<section>.xml', views.sitemap,
         {'sitemaps': secure_sitemaps},
         name='django.contrib.sitemaps.views.sitemap'),
 ]
