@@ -31,8 +31,8 @@ class StaticFilesFormsMediaTestCase(SimpleTestCase):
             str(m),
             """<link href="https://example.com/assets/path/to/css1" type="text/css" media="all" rel="stylesheet">
 <link href="/path/to/css2" type="text/css" media="all" rel="stylesheet">
-<script type="text/javascript" src="/path/to/js1"></script>
-<script type="text/javascript" src="http://media.other.com/path/to/js2"></script>
-<script type="text/javascript" src="https://secure.other.com/path/to/js3"></script>
-<script type="text/javascript" src="https://example.com/assets/relative/path/to/js4"></script>"""
+<script src="/path/to/js1"></script>
+<script src="http://media.other.com/path/to/js2"></script>
+<script src="https://secure.other.com/path/to/js3"></script>
+<script src="https://example.com/assets/relative/path/to/js4"></script>"""
         )
