@@ -29,6 +29,9 @@ class TestRss2Feed(views.Feed):
     def item_updateddate(self, item):
         return item.updated
 
+    def item_comments(self, item):
+        return "%scomments" % item.get_absolute_url()
+
     item_author_name = 'Sally Smith'
     item_author_email = 'test@example.com'
     item_author_link = 'http://www.example.com/'
@@ -74,7 +77,7 @@ class TestLatestFeed(TestRss2Feed):
     subtitle = TestRss2Feed.description
 
     def items(self):
-        return Entry.objects.exclude(pk=5)
+        return Entry.objects.exclude(title='My last entry')
 
 
 class ArticlesFeed(TestRss2Feed):
@@ -136,6 +139,30 @@ class TemplateContextFeed(TestRss2Feed):
         return context
 
 
+class TestLanguageFeed(TestRss2Feed):
+    language = 'de'
+
+
+class TestGetObjectFeed(TestRss2Feed):
+    def get_object(self, request, entry_id):
+        return Entry.objects.get(pk=entry_id)
+
+    def items(self, obj):
+        return Article.objects.filter(entry=obj)
+
+    def item_link(self, item):
+        return '%sarticle/%s/' % (item.entry.get_absolute_url(), item.pk)
+
+    def item_comments(self, item):
+        return '%scomments' % self.item_link(item)
+
+    def item_description(self, item):
+        return 'Article description: %s' % item.title
+
+    def item_title(self, item):
+        return 'Title: %s' % item.title
+
+
 class NaiveDatesFeed(TestAtomFeed):
     """
     A feed with naive (non-timezone-aware) dates.
