@@ -4,6 +4,7 @@ from . import feeds
 
 urlpatterns = [
     path('syndication/rss2/', feeds.TestRss2Feed()),
+    path('syndication/rss2/articles/<int:entry_id>/', feeds.TestGetObjectFeed()),
     path(
         'syndication/rss2/guid_ispermalink_true/',
         feeds.TestRss2FeedWithGuidIsPermaLinkTrue()),
@@ -15,6 +16,7 @@ urlpatterns = [
     path('syndication/atom/', feeds.TestAtomFeed()),
     path('syndication/latest/', feeds.TestLatestFeed()),
     path('syndication/custom/', feeds.TestCustomFeed()),
+    path('syndication/language/', feeds.TestLanguageFeed()),
     path('syndication/naive-dates/', feeds.NaiveDatesFeed()),
     path('syndication/aware-dates/', feeds.TZAwareDatesFeed()),
     path('syndication/feedurl/', feeds.TestFeedUrlFeed()),
