@@ -1,4 +1,5 @@
 from django.template import TemplateSyntaxError
+from django.template.defaulttags import ForNode
 from django.test import SimpleTestCase
 
 from ..utils import setup
@@ -180,3 +181,38 @@ class ForTagTests(SimpleTestCase):
     def test_for_tag_unpack14(self):
         with self.assertRaisesMessage(ValueError, 'Need 2 values to unpack in for loop; got 1.'):
             self.engine.render_to_string('for-tag-unpack14', {'items': (1, 2)})
+
+    @setup({
+        'main': '{% with alpha=alpha.values %}{% include "base" %}{% endwith %}_'
+                '{% with alpha=alpha.extra %}{% include "base" %}{% endwith %}',
+        'base': '{% for x, y in alpha %}{{ x }}:{{ y }},{% endfor %}'
+    })
+    def test_for_tag_context(self):
+        """
+        ForNode.render() pops the values it pushes to the context (#28001).
+        """
+        output = self.engine.render_to_string('main', {
+            'alpha': {
+                'values': [('two', 2), ('four', 4)],
+                'extra': [('six', 6), ('eight', 8)],
+            },
+        })
+        self.assertEqual(output, 'two:2,four:4,_six:6,eight:8,')
+
+    @setup({'invalid_for_loop': '{% for x items %}{{ x }}{% endfor %}'})
+    def test_invalid_arg(self):
+        msg = "'for' statements should have at least four words: for x items"
+        with self.assertRaisesMessage(TemplateSyntaxError, msg):
+            self.engine.render_to_string('invalid_for_loop', {'items': (1, 2)})
+
+    @setup({'invalid_for_loop': '{% for x from items %}{{ x }}{% endfor %}'})
+    def test_invalid_in_keyword(self):
+        msg = "'for' statements should use the format 'for x in y': for x from items"
+        with self.assertRaisesMessage(TemplateSyntaxError, msg):
+            self.engine.render_to_string('invalid_for_loop', {'items': (1, 2)})
+
+
+class ForNodeTests(SimpleTestCase):
+    def test_repr(self):
+        node = ForNode('x', 'sequence', is_reversed=True, nodelist_loop=['val'], nodelist_empty=['val2'])
+        self.assertEqual(repr(node), '<ForNode: for x in sequence, tail_len: 1 reversed>')
