@@ -14,7 +14,15 @@ urlpatterns = [
     path('raw_post_view/', views.raw_post_view),
     path('redirect_view/', views.redirect_view),
     path('redirect_view_307/', views.method_saving_307_redirect_view),
+    path(
+        'redirect_view_307_query_string/',
+        views.method_saving_307_redirect_query_string_view,
+    ),
     path('redirect_view_308/', views.method_saving_308_redirect_view),
+    path(
+        'redirect_view_308_query_string/',
+        views.method_saving_308_redirect_query_string_view,
+    ),
     path('secure_view/', views.view_with_secure),
     path('permanent_redirect_view/', RedirectView.as_view(url='/get_view/', permanent=True)),
     path('temporary_redirect_view/', RedirectView.as_view(url='/get_view/', permanent=False)),
@@ -44,4 +52,6 @@ urlpatterns = [
     path('accounts/no_trailing_slash', RedirectView.as_view(url='login/')),
     path('accounts/login/', auth_views.LoginView.as_view(template_name='login.html')),
     path('accounts/logout/', auth_views.LogoutView.as_view()),
+    # Async views.
+    path('async_get_view/', views.async_get_view, name='async_get_view'),
 ]
