@@ -4,21 +4,13 @@ from django.db import models
 class Car(models.Model):
     name = models.CharField(max_length=100)
 
-    def __str__(self):
-        return self.name
-
 
 class Person(models.Model):
     name = models.CharField(max_length=100)
     cars = models.ManyToManyField(Car, through='PossessedCar')
-
-    def __str__(self):
-        return self.name
+    data = models.BinaryField(null=True)
 
 
 class PossessedCar(models.Model):
     car = models.ForeignKey(Car, models.CASCADE)
-    belongs_to = models.ForeignKey(Person, models.CASCADE)
-
-    def __str__(self):
-        return self.color
+    belongs_to = models.ForeignKey(Person, models.CASCADE, related_name='possessed_cars')
