@@ -11,17 +11,11 @@ class DataPoint(models.Model):
     value = models.CharField(max_length=20)
     another_value = models.CharField(max_length=20, blank=True)
 
-    def __str__(self):
-        return self.name
-
 
 class RelatedPoint(models.Model):
     name = models.CharField(max_length=20)
     data = models.ForeignKey(DataPoint, models.CASCADE)
 
-    def __str__(self):
-        return self.name
-
 
 class A(models.Model):
     x = models.IntegerField(default=10)
@@ -47,3 +41,11 @@ class Foo(models.Model):
 class Bar(models.Model):
     foo = models.ForeignKey(Foo, models.CASCADE, to_field='target')
     m2m_foo = models.ManyToManyField(Foo, related_name='m2m_foo')
+
+
+class UniqueNumber(models.Model):
+    number = models.IntegerField(unique=True)
+
+
+class UniqueNumberChild(UniqueNumber):
+    pass
