@@ -10,7 +10,8 @@ class TestHashable(SimpleTestCase):
             ({}, ()),
             ({'a'}, ('a',)),
             (frozenset({'a'}), {'a'}),
-            ({'a': 1}, (('a', 1),)),
+            ({'a': 1, 'b': 2}, (('a', 1), ('b', 2))),
+            ({'b': 2, 'a': 1}, (('a', 1), ('b', 2))),
             (('a', ['b', 1]), ('a', ('b', 1))),
             (('a', {'b': 1}), ('a', (('b', 1),))),
         )
