@@ -1,18 +1,15 @@
-from __future__ import unicode_literals
-
 import copy
 import pickle
 import sys
 import warnings
 from unittest import TestCase
 
-from django.utils import six
 from django.utils.functional import LazyObject, SimpleLazyObject, empty
 
 from .models import Category, CategoryInfo
 
 
-class Foo(object):
+class Foo:
     """
     A simple class with just one attribute.
     """
@@ -69,13 +66,23 @@ class LazyObjectTestCase(TestCase):
         self.assertNotEqual(obj1, obj2)
         self.assertNotEqual(obj1, 'bar')
 
+    def test_lt(self):
+        obj1 = self.lazy_wrap(1)
+        obj2 = self.lazy_wrap(2)
+        self.assertLess(obj1, obj2)
+
+    def test_gt(self):
+        obj1 = self.lazy_wrap(1)
+        obj2 = self.lazy_wrap(2)
+        self.assertGreater(obj2, obj1)
+
     def test_bytes(self):
         obj = self.lazy_wrap(b'foo')
         self.assertEqual(bytes(obj), b'foo')
 
     def test_text(self):
         obj = self.lazy_wrap('foo')
-        self.assertEqual(six.text_type(obj), 'foo')
+        self.assertEqual(str(obj), 'foo')
 
     def test_bool(self):
         # Refs #21840
@@ -170,7 +177,7 @@ class LazyObjectTestCase(TestCase):
         # Tests whether an object's custom `__iter__` method is being
         # used when iterating over it.
 
-        class IterObject(object):
+        class IterObject:
 
             def __init__(self, values):
                 self.values = values
@@ -304,7 +311,7 @@ class SimpleLazyObjectTestCase(LazyObjectTestCase):
         obj = self.lazy_wrap(42)
         # __repr__ contains __repr__ of setup function and does not evaluate
         # the SimpleLazyObject
-        six.assertRegex(self, repr(obj), '^<SimpleLazyObject:')
+        self.assertRegex(repr(obj), '^<SimpleLazyObject:')
         self.assertIs(obj._wrapped, empty)  # make sure evaluation hasn't been triggered
 
         self.assertEqual(obj, 42)  # evaluate the lazy object
@@ -361,7 +368,7 @@ class SimpleLazyObjectTestCase(LazyObjectTestCase):
         self.assertEqual(len(lazy_set), 4)
 
 
-class BaseBaz(object):
+class BaseBaz:
     """
     A base class with a funky __reduce__ method, meant to simulate the
     __reduce__ method of Model, which sets self._django_version.
@@ -371,7 +378,7 @@ class BaseBaz(object):
 
     def __reduce__(self):
         self.baz = 'right'
-        return super(BaseBaz, self).__reduce__()
+        return super().__reduce__()
 
     def __eq__(self, other):
         if self.__class__ != other.__class__:
@@ -390,11 +397,11 @@ class Baz(BaseBaz):
     """
     def __init__(self, bar):
         self.bar = bar
-        super(Baz, self).__init__()
+        super().__init__()
 
     def __reduce_ex__(self, proto):
         self.quux = 'quux'
-        return super(Baz, self).__reduce_ex__(proto)
+        return super().__reduce_ex__(proto)
 
 
 class BazProxy(Baz):
@@ -406,6 +413,7 @@ class BazProxy(Baz):
     def __init__(self, baz):
         self.__dict__ = baz.__dict__
         self._baz = baz
+        # Grandparent super
         super(BaseBaz, self).__init__()
 
 
