@@ -1,7 +1,7 @@
-from django.conf.urls import url
 from django.contrib import admin
+from django.urls import path
 
 urlpatterns = [
     # This is the same as in the default project template
-    url(r'^admin/', admin.site.urls),
+    path('admin/', admin.site.urls),
 ]
