@@ -23,21 +23,21 @@ class DefaultsTests(TestCase):
 
     @classmethod
     def setUpTestData(cls):
-        Author.objects.create(name='Boris')
+        author = Author.objects.create(name='Boris')
         Article.objects.create(
-            title='Old Article', slug='old_article', author_id=1,
+            title='Old Article', slug='old_article', author=author,
             date_created=datetime.datetime(2001, 1, 1, 21, 22, 23)
         )
         Article.objects.create(
-            title='Current Article', slug='current_article', author_id=1,
+            title='Current Article', slug='current_article', author=author,
             date_created=datetime.datetime(2007, 9, 17, 21, 22, 23)
         )
         Article.objects.create(
-            title='Future Article', slug='future_article', author_id=1,
+            title='Future Article', slug='future_article', author=author,
             date_created=datetime.datetime(3000, 1, 1, 21, 22, 23)
         )
-        UrlArticle.objects.create(
-            title='Old Article', slug='old_article', author_id=1,
+        cls.urlarticle = UrlArticle.objects.create(
+            title='Old Article', slug='old_article', author=author,
             date_created=datetime.datetime(2001, 1, 1, 21, 22, 23)
         )
         Site(id=1, domain='testserver', name='testserver').save()
@@ -47,6 +47,11 @@ class DefaultsTests(TestCase):
         for url in self.nonexistent_urls:
             response = self.client.get(url)
             self.assertEqual(response.status_code, 404)
+        self.assertIn(b'<h1>Not Found</h1>', response.content)
+        self.assertIn(
+            b'<p>The requested resource was not found on this server.</p>',
+            response.content,
+        )
 
     @override_settings(TEMPLATES=[{
         'BACKEND': 'django.template.backends.django.DjangoTemplates',
@@ -71,13 +76,12 @@ class DefaultsTests(TestCase):
     def test_server_error(self):
         "The server_error view raises a 500 status"
         response = self.client.get('/server_error/')
-        self.assertEqual(response.status_code, 500)
+        self.assertContains(response, b'<h1>Server Error (500)</h1>', status_code=500)
 
     def test_bad_request(self):
         request = self.request_factory.get('/')
         response = bad_request(request, Exception())
-        self.assertEqual(response.status_code, 400)
-        self.assertEqual(response.content, b'<h1>Bad Request (400)</h1>')
+        self.assertContains(response, b'<h1>Bad Request (400)</h1>', status_code=400)
 
     @override_settings(TEMPLATES=[{
         'BACKEND': 'django.template.backends.django.DjangoTemplates',
@@ -107,7 +111,7 @@ class DefaultsTests(TestCase):
         "A model can set attributes on the get_absolute_url method"
         self.assertTrue(getattr(UrlArticle.get_absolute_url, 'purge', False),
                         'The attributes of the original get_absolute_url must be added.')
-        article = UrlArticle.objects.get(pk=1)
+        article = UrlArticle.objects.get(pk=self.urlarticle.pk)
         self.assertTrue(getattr(article.get_absolute_url, 'purge', False),
                         'The attributes of the original get_absolute_url must be added.')
 
@@ -129,3 +133,18 @@ class DefaultsTests(TestCase):
 
         with self.assertRaises(TemplateDoesNotExist):
             server_error(request, template_name='nonexistent')
+
+    def test_error_pages(self):
+        request = self.request_factory.get('/')
+        for response, title in (
+            (bad_request(request, Exception()), b'Bad Request (400)'),
+            (permission_denied(request, Exception()), b'403 Forbidden'),
+            (page_not_found(request, Http404()), b'Not Found'),
+            (server_error(request), b'Server Error (500)'),
+        ):
+            with self.subTest(title=title):
+                self.assertIn(b'<!doctype html>', response.content)
+                self.assertIn(b'<html lang="en">', response.content)
+                self.assertIn(b'<head>', response.content)
+                self.assertIn(b'<title>%s</title>' % title, response.content)
+                self.assertIn(b'<body>', response.content)
